#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
'''
Created on 1 janv. 2015

@author: jerome
'''
from pyromaths import ex
from random import randrange
from math import sqrt
from pyromaths.outils import Priorites3
from pyromaths.classes.PolynomesCollege import Polynome, Lagrange
from pyromaths.classes.Fractions import Fraction

if __name__ == '__main__':
    from pyromaths.outils import Priorites3

    pass

def racines(p, lracines):
    """ Trouve les 2 racines manquantes d'un polynôme de degré 5 connaissant 3 des racines
    """
    q = Polynome(p)
    for r in lracines:
        q = q // Polynome("x-%s" % r)
    delta = q[1][0] ** 2 - 4 * q[0][0] * q[2][0]
    d = float(eval(Priorites3.priorites(delta)[-1][0]))
    return (-float(q[1][0]) + sqrt(d)) / 2 / float(q[0][0]), (-float(q[1][0]) - sqrt(d)) / 2 / float(q[0][0])

class _EquationGraphique(ex.TexExercise):
    """Résolution graphique d'équations"""

    tags = ["Seconde"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        """
        l1, l2 et l3 sont les abscisses remarquables,
        y1, y2 et y3 sont les ordonnées correspondantes

        l1 contient trois abscisses de même ordonnée
        l2 contient deux abscisses de même ordonnée
        l3 contient 2 abscisses séparées d'un dixième (un extremum local j'espère) de même ordonnée
        """
        End = False
        while End == False:
            encore = True
            while encore:
                l1 = [a for a in range(-6, 7)]
                for dummy in range(10):
                    l1.pop(randrange(len(l1)))
                if l1[2] - l1[1] > 3 and l1[1] - l1[0] > 3: encore = False
            pos = randrange(2)
            l2 = randrange(1, 4) / 2
            l2 = [l1[pos] + l2, l1[pos + 1] - l2]
            l3 = [(l1[(pos + 1) % 2] + l1[(pos + 1) % 2 + 1]) / 2]
            l1.extend(l2)
            l1.extend(l3)
            y = [randrange(2) * (-1) ** randrange(2), randrange(3, 5) * (-1) ** randrange(2)]
            # if randrange(2): y[0], y[2] = y[2], y[0]
            #=======================================================================
            # y = [y[1], y[1], y[1], y[0], y[0], y[2], y[2]]
            #=======================================================================
            y = [y[1], y[1], y[1], y[0], y[0], y[0]]
            self.points = list([(l1[i], y[i]) for i in range(6)])
            p = Lagrange(self.points)
            self.polynome = eval(Priorites3.priorites(p)[-1][0])
            End = True
            for val in range(13):
                #===============================================================
                # try:
                #     ordonnee = eval(self.polynome(val - 6))
                # except:
                #     print repr(self.polynome), val - 6
                # if isinstance(ordonnee, str):
                #     ordonnee = eval(Priorites3.priorites(ordonnee)[-1][0])
                #===============================================================
                try:
                    ordonnee = eval(Priorites3.priorites(self.polynome(val - 6))[-1][0])
                except IndexError:
                    ordonnee = eval(self.polynome(val - 6))
                if ordonnee > 5 or ordonnee < -5:
                    End = False
                    break

    def tex_statement(self):
        exo = [r'\exercice']
        exo.extend([r"\psset{unit=5mm, algebraic, dotsize=4pt 4}", r"\begin{pspicture*}(-6.2,-5.2)(6.2,5.2)"])
        exo.append(r"\psgrid[subgriddiv=1, gridwidth=.6pt,subgridcolor=lightgray, gridlabels=0pt]")
        exo.append(r"\psaxes[linewidth=1.2pt,]{->}(0,0)(-6.2,-5.2)(6.2,5.2)")
        exo.append(r"\psplot[plotpoints=200, linewidth=1.5pt, linecolor=DarkRed]{-6}{6}{%s}" % Priorites3.plotify(repr(self.polynome)))
        exo.append(r"\psdots %s" % " ".join([str(val) for val in self.points]))
        exo.append(r"\end{pspicture*}")
        #=======================================================================
        # print racines(self.polynome, [self.points[i][0] for i in range(3)])
        # print racines(self.polynome, [self.points[i][0] for i in range(3, 6)])
        #=======================================================================
        return "\n".join(exo)



#!/usr/bin/env python3

# Copyright (C) 2015 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""Test of exercises.

This module gather tests from all exercises. Running:

    python3 -m unittest discover

does just as expected.
"""
import codecs
import difflib
import gettext
import glob
import logging
import os
import random
import shutil
import tempfile
import unittest

# Quick and dirty definition of `_` as the identity function
gettext.install('pyromaths')

import pyromaths
from pyromaths.outils import System
from pyromaths.cli import PyromathsException
from pyromaths import directories

# Logging configuration
logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger()

def load_tests(*__args, **__kwargs):
    """Return an `unittest.TestSuite` containing tests from all exercises."""
    tests = TestPerformer()
    return tests.as_unittest_suite([
        (exo, tests.get_tested_seeds(exo))
        for exo in tests.iter_names()
        ])

class ExerciseNotFound(PyromathsException):
    """Name of exercise cannot be found in known exercises."""

    def __init__(self, exercise):
        super().__init__()
        self.exercise = exercise

    def __str__(self):
        return "Exercise '{}' not found.".format(self.exercise)

def test_path(name, seed, choice):
    """Return the path of file containing expected test result."""
    return os.path.join(directories.EXODIR, 'tests', "%s.%s.%s" % (name, seed, choice))

class UnittestExercise(unittest.TestCase):
    """Test an exercise, with a particular seed."""

    maxDiff = None

    def __init__(self, exercise=None):
        super().__init__()
        self.exercise = exercise

    def shortDescription(self):
        if self.exercise is None:
            return super().shortDescription()
        else:
            return "{}-{}".format(self.exercise.name(), self.exercise.seed)

    def runTest(self):
        """Perform test"""
        self.assertEqual(
            self.exercise.tex_statement(),
            self.exercise.read_test('statement'),
            )

        self.assertEqual(
            self.exercise.tex_answer(),
            self.exercise.read_test('answer'),
            )

class TestPerformer:
    """Perform tests over every exercises"""

    def __init__(self):
        self.exercises = pyromaths.ex.ExerciseBag()

    def iter_names(self):
        """Iterate over exercise names."""
        return list(self.exercises.keys())

    def get(self, exercise, seed):
        """Return the `TestExercise` object corresponding to the arguments."""
        if exercise not in self.exercises:
            raise KeyError(exercise)
        return self.exercises[exercise](seed)

    def get_tested_seeds(self, exercise):
        """Return seeds that are tested for this exercise"""
        if exercise not in self.exercises:
            raise ExerciseNotFound(exercise)
        statement_seeds = [
            os.path.basename(path).split(".")[1]
            for path in glob.glob(test_path(
                self.exercises[exercise].name(),
                '*',
                'statement'
                ))
            ]
        for seed in statement_seeds:
            if os.path.exists(test_path(
                    self.exercises[exercise].name(),
                    seed,
                    'answer',
                )):
                yield int(seed)

    def iter_missing(self):
        """Iterate over exercises that are not tested."""
        for exercise in self.exercises:
            if not list(self.get_tested_seeds(exercise)):
                yield exercise

    def as_unittest_suite(self, exercises):
        """Return the tests, as a `unittest.TestSuite`."""
        suite = unittest.TestSuite()
        for exercise, seeds in exercises:
            for seed in seeds:
                suite.addTest(UnittestExercise(exercise=self.get(exercise, seed)))
        return suite



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2014 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from random import randrange, shuffle
from pyromaths.classes.Fractions import Fraction
from pyromaths.outils import Priorites3
from pyromaths.outils.Arithmetique import pgcd
from pyromaths.ex import LegacyExercise


def _id_rem():
    """Génère un exercice de développement des 3 identités remarquables avec une situation piège.
    Dans un premier temps, on n'utilise que des nombres entiers, puis des fractions, puis l'opposé 
    d'une expression littérale.
    """

    l = [randrange(1, 11) for dummy in range(14)]
    while pgcd(l[8], l[9]) != 1 or pgcd(l[10], l[11]) != 1 or (l[9] == 1 and l[11] == 1):
        # On crée deux rationnels irréductibles non tous deux entiers.
        l = [randrange(1, 11) for dummy in range(14)]
    lpoly = [id_rem1(l[0], l[1]), id_rem2(l[2], l[3]), id_rem3(l[4], l[5]), id_rem4(l[6], l[7])]
    shuffle(lpoly)
    lid = [id_rem1, id_rem2, id_rem3, id_rem4]
    lpoly2 = [lid.pop(randrange(4))(Fraction(l[8], l[9]), Fraction(l[10], l[11]))]
    lpoly2.append('-' + lid.pop(randrange(3))(l[12], l[13]))
    shuffle(lpoly2)
    lpoly.extend(lpoly2)
    expr = [Priorites3.texify([Priorites3.splitting(lpoly[i])]) for i in range(6)]
    exo = ["\\exercice", u"Développer chacune des expressions littérales suivantes :"]
    exo.append("\\begin{multicols}{2}")
    exo.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), expr[i][0]) for i in range(6)]))
    exo.append("\\end{multicols}")
    cor = ["\\exercice*", u"Développer chacune des expressions littérales suivantes :"]
    cor.append("\\begin{multicols}{2}")
    for i in range(6):
        dev = Priorites3.texify(Priorites3.priorites(lpoly[i]))
        dev.insert(0, expr[i][0])
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), dev[j]) for j in range(len(dev) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), dev[-1]))
    cor.append("\\end{multicols}")

    return exo, cor

def id_rem1(a, b, details=2):
    """Construit un Polynome de la forme (ax+b)^2
    Renvoie une chaine"""
    return 'Polynome([[%r, 1], [%r, 0]], details=%s)**2' % (a, b, details)
def id_rem2(a, b, details=2):
    """Construit un Polynome de la forme (ax-b)^2
    Renvoie une chaine"""
    return 'Polynome([[%r, 1], [%r, 0]], details=%s)**2' % (a, -b, details)
def id_rem3(a, b, details=2):
    """Construit un Polynome de la forme (ax+b)(ax-b) ou (ax-b)(ax+b)
    Renvoie une chaine"""
    sgn = randrange(2)
    return 'Polynome([[%r, 1], [%r, 0]], details=%s) * Polynome([[%r, 1], [%r, 0]], details=%s)' % (a, (-1) ** sgn * b, details, a, (-1) ** (sgn + 1) * b, details)
def id_rem4(a, b, details=2):
    """Construit un Polynome de la forme (ax+b)(bx-a) ou (ax-b)(bx+a)
    Renvoie une chaine"""
    sgn = randrange(2)
    return 'Polynome([[%r, 1], [%r, 0]], details=%s) * Polynome([[%r, 1], [%r, 0]], details=%s)' % (a, (-1) ** sgn * b, details, b, (-1) ** (sgn + 1) * a, details)


class id_rem(LegacyExercise):
    """Identités remarquables"""

    tags = ["Troisième"]
    function = _id_rem



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

# import re
from pyromaths.classes import Racine
from pyromaths.classes.Fractions import Fraction

from .decimaux import decimaux


def tex_coef(coef, var='', bplus=0, bpn=0, bpc=0):
    r"""**tex_coef**\ (*coef*\ [, *var*\ [, *bplus*\ [, *bpn*\ [, *bpc*\ ]]]])

    Gère l'affichage d'un monôme (par défaut de degré 0) au format TeX. Permet

    * d'écrire `3x` au format TeX
    * d'ajouter un `+` devant `3x` pour écrire la somme `2+3x` avec `bplus`
    * de mettre des parenthèses autour de `-3x` pour écrire le produit `(x+1)(-3x)` avec `bpn` alors qu'il n'en faut pas avec `(x+1)3x`
    * de mettre des parenthèses autour de 3x pour écrire la puissance `(3x)^2` avec `bpc` alors qu'il n'en faut pas pour `x^2`

    :param coef: est le coefficient à écrire devant la variable var
    :type coef: integer
    :param bplus: si vrai, il faut écrire le signe + devant le coefficient
    :type bplus: boolean
    :param bpn: si vrai, il faut mettre des parenthèses autour de l'écriture si *coef* est négatif.
    :type bpn: boolean
    :param bpc: si vrai, il faut mettre des parenthèses autour de l'écriture si `coef != 0 ou 1` et `var != ''`
    :type bpc: boolean

    >>> from pyromaths.outils import Affichage
    >>> Affichage.tex_coef(5,'x')
    5\,x
    >>> Affichage.tex_coef(5,'x',1)
    +5\,x
    >>> Affichage.tex_coef(-5,'x',0,1)
    \left( -5\,x\right)
    >>> Affichage.tex_coef(1,'x',0,1,1)
    x
    >>> Affichage.tex_coef(5,'x',0,1,1)
    \left( 5\,x\right)

    :rtype: string
    """
    if coef != 0 and abs(coef) != 1:
        if var == '':
                a = TeX(coef)
        else:
                a = '%s\\,%s' % (TeX(coef), var)
        if bplus and coef > 0:
            a = '+' + a
    elif coef == 1:
        if var == '':
            a = '1'
        else:
            a = '%s' % var
        if bplus:
            a = '+' + a
    elif coef == 0:
        a = ''
    elif coef == -1:
        if var == '':
            a = '-1'
        else:
            a = '-%s' % var
    if bpn and coef < 0 or bpc and coef != 0 and coef != 1 and var != '':
        a = '\\left( ' + a + '\\right)'
    return a

def TeXz(nombre):
    """n'affiche pas b si b=0

    Double emploi avec tex_coef(nombre)"""
    if nombre == 0:
        return ""
    else:
        return TeX(nombre)
def tTeX(nombre):
    '''raccourci pour TeX(nombre,terme=True)'''
    return TeX(nombre, terme=True)
def pTeX(nombre):
    '''raccourci pour TeX(nombre,parenthese=True)'''
    return TeX(nombre, parenthese=True)
def fTeX(nombre):
    '''raccourci pour TeX(nombre,fractex="\\frac")'''
    return TeX(nombre, fractex="\\frac")

def TeX(nombre, parenthese=False, terme=False, fractex="\\dfrac"):
    r"""**TeX**\ (*nombre*\ [, *parenthese*\ [, *terme*\ [, *fractex*\ ]]])

    Permet l'affichage de nombres au format TeX. Fait en partie double emploi avec tex_coef. Permet

    * d'écrire un nombre (décimal, rationnel ou radical) au format TeX
    * d'écrire l'infini au format TeX
    * d'ajouter un `+` devant `3` pour écrire `+3` avec `terme`
    * de mettre des parenthèses autour de `3` avec `parenthese`
    * de changer la commande d'écriture des fractions en TeX avec fractex

    :param nombre: est le nombre à écrire
    :type nombre: integer, Fractions, RacineDegre2
    :param parenthese: si vrai, il faut écrire des parenthèses autour de `nombre`
    :type parenthese: boolean
    :param terme: Ajoute un `+` devant nombre s'il est positif.
    :type terme: boolean
    :param fractex: commande à utiliser pour créer une fraction sous TeX
    :type fractex: string

    >>> from pyromaths.outils import Affichage
    >>> from pyromaths.classes.Fractions import Fraction
    >>> Affichage.TeX(Fraction(7,3)).strip()
    \dfrac{7}{3}
    >>> Affichage.TeX(Fraction(7,3),fractex='\\frac').strip()
    \frac{7}{3}

    :rtype: string
    """
    strTeX = finTeX = ""

    # Affichage simplifié des racines ou fractions
    if isinstance(nombre, Racine.RacineDegre2) and nombre.radicande == 0:
        # Affiche la RacineDegre2 comme une Fractions
        nombre = Fraction(nombre.numerateur, nombre.denominateur)
    if isinstance(nombre, Fraction) and nombre.d == 1:
        # Affiche la Fractions comme un entier
        nombre = nombre.n
    # parentheses des fractions
    if parenthese and (
        isinstance(nombre, Racine.RacineDegre2)
                       and nombre.denominateur == 1 and (nombre.numerateur or nombre.coeff < 0)
        # RacineDegre2 avec radicande nécessairement grâce au tri
        or isinstance(nombre, Fraction) and nombre.n < 0
        or isinstance(nombre, int) and nombre < 0
        or isinstance(nombre, float) and nombre < 0):
        strTeX = "\\left("
        finTeX = "\\right)"
    elif terme and (isinstance(nombre, Racine.RacineDegre2) and
                        (nombre.d != 1 or (nombre.n > 0 or nombre.n == 0 and nombre.coeff >= 0))
                    or nombre >= 0) :
        strTeX = "+"
        finTeX = ""

    # #Affichage
    if isinstance(nombre, (int, float)) and nombre == float("inf"):
        return "+\\infty "
    elif isinstance(nombre, (int, float)) and nombre == float("-inf"):
        return "-\\infty "
    elif isinstance(nombre, int) or isinstance(nombre, float):
        return strTeX + decimaux(nombre) + finTeX
    elif isinstance(nombre, Fraction):
        if nombre.n < 0:
            strTeX += "-" + fractex + "{" + decimaux(-nombre.n) + "}{" + decimaux(nombre.d) + "} "
        else:
            strTeX += fractex + "{" + decimaux(nombre.n) + "}{" + decimaux(nombre.d) + "} "
        strTeX += finTeX
        return strTeX
    elif isinstance(nombre, Racine.RacineDegre2):
        return strTeX + str(nombre) + finTeX
    else:
        return strTeX + str(nombre) + finTeX

def radicalTeX(n):
    return "\\sqrt{%s}" % (decimaux(n))



#!/usr/bin/env python3

# Copyright (C) 2016-2018 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""Pyromaths command line interface.

To display help:

> python3 -m pyromaths --help
"""
import argparse
import gettext
import logging
import random
import re
import shutil
import subprocess
import sys
import textwrap

import gettext

from pyromaths import directories
gettext.bindtextdomain('pyromaths', directories.LOCALEDIR)
gettext.textdomain('pyromaths')
_ = gettext.gettext

from pyromaths.cli import exercise_argument, PyromathsException
from pyromaths.ex import ExerciseBag
from pyromaths.outils.System import Fiche
from pyromaths.version import VERSION

# Logging configuration
logging.basicConfig(level=logging.INFO)
LOGGER = logging.getLogger()

def _tag_type(string):
    return set(tag for tag in string.split("+") if tag)

ALLOWED_FORMATS = ('tex', 'pdf', 'latexmk')

def _format_type(string):
    formats = string.split(",")
    for chunk in formats:
        if chunk not in ALLOWED_FORMATS:
            raise argparse.ArgumentTypeError("Format '{}' must be one of {}.".format(
                chunk,
                ", ".join("'{}'".format(frmt) for frmt in ALLOWED_FORMATS),
                ))
    return formats

def argument_parser():
    """Return an argument parser"""
    parser = argparse.ArgumentParser(
        prog='pyromaths',
        )
    parser.add_argument(
        '-v', '--version',
        action='version',
        version='%(prog)s {version}'.format(version=VERSION),
        )
    subparsers = parser.add_subparsers(title='Commands', dest='command')
    subparsers.required = True
    subparsers.dest = 'command'

    # List exos
    ls = subparsers.add_parser( # pylint: disable=unused-variable
        'ls',
        help=(
            "List available exercises. Each line of the output can be used as "
            "an argument to other commands."
            ),
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=textwrap.dedent("""\
                # Filtrer les exercices

                Les options --tags et --desc permettent de filtrer quels exercices afficher. Par exemple, pour n'afficher que les exercices de seconde dont la description contient le mot « variations », utiliser :

                    pyromaths ls --desc variation --tag +Seconde
                """),
        )
    ls.add_argument(
        "-v", "--verbose",
        help="Affiche davantage de détails.",
        action="store_true",
        )
    ls.add_argument(
        "-d", "--desc",
        help=textwrap.dedent("""\
               Ne liste que les exercice dont la description correspond à l'argument de cette fonction, considéré comme une expression régulière.
               Si plusieurs arguments --desc sont donnés, liste les exercices correspondant à l'un des arguments.
               """),
        action="append",
        default=list(),
        type=re.compile,
        )
    ls.add_argument(
        "-t", "--tags",
        help=textwrap.dedent("""\
               Ne liste que les exercice taggés avec tous des tags fournis en argument (sous la forme "+tag1+tag2+tag3").
               Si plusieurs arguments --tags sont donnés, liste les exercices vérifiant l'un des arguments --tags.
               La liste des tags peut être affichée avec la commande `pyromaths tags`.
               """),
        action="append",
        default=list(),
        type=_tag_type,
        )

    # List tags
    ls = subparsers.add_parser( # pylint: disable=unused-variable
        'tags',
        help="List available tags.",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )

    # Generate
    generate_parser = subparsers.add_parser(
        'generate',
        help='Generate some exercises.',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )
    generate_parser.add_argument(
        "exercise",
        metavar='EXERCISE[:SEED[,SEED]]',
        nargs='+', type=exercise_argument, default=None,
        help='Exercises to generate.'
        )
    generate_parser.add_argument(
        '-p', '--pipe',
        nargs=1,
        type=str,
        action='append',
        help=(
            "Commands to run on the LaTeX file before compiling. String '{}' "
            "is replaced by the file name; if not, it is appended at the end "
            "of the string."
            )
        )
    generate_parser.add_argument(
        '-d', '--dirty',
        action='store_const',
        const=True,
        default=False,
        help=(
            "Do not clean temporary directory after compilation."
            )
        )
    generate_parser.add_argument(
        '-o', '--output',
        type=str,
        default='exercice',
        help=(
            "Output filename (without extension). Default is 'exercice'."
            ),
        )
    generate_parser.add_argument(
        '-f', '--format',
        default='pdf',
        help=textwrap.dedent("""\
            Format de l'exercice à générer, parmi "tex" (source au format LaTeX), "pdf" (exercice compilé), "latexmkrc" (fichier de configuration utilisé pour la compilation du fichier LaTeX). Il est possible de générer plusieurs formats séparés par des virgules, comme par exemple --format=tex,pdf.
            """),
        type=_format_type,
        )

    # Test
    test = subparsers.add_parser(
        'test',
        help='Test exercices',
        )
    test.add_argument('args', nargs=argparse.REMAINDER)

    # Dummy
    dummy = subparsers.add_parser(
        'dummy',
        help='Generate a dummy LaTeX file.',
        )
    dummy.add_argument('args', nargs=argparse.REMAINDER)

    return parser

def do_test(options):
    """Action for command line 'test'."""
    from pyromaths.cli.test import __main__
    sys.exit(__main__.main(options.args))

def do_dummy(options):
    """Action for command line 'dummy'."""
    from .cli import dummy
    dummy.main()

def do_generate(options):
    """Action for command line 'generate'."""

    if options.pipe is None:
        options.pipe = []
    else:
        options.pipe = [item[0] for item in options.pipe]

    bag = ExerciseBag()
    exercise_list = []

    for exercise, seeds in options.exercise:
        if not seeds:
            seeds = [random.randint(0, sys.maxsize)]
        for seed in seeds:
            exercise_list.append(bag[exercise](seed))

        exercise_list,
    parametres = {
        'enonce': True,
        'corrige': True,
        'exercices': exercise_list,
        }
    with Fiche(parametres, dirty=options.dirty) as fiche:
        # LatexmkRC
        if 'latexmk' in options.format:
            fiche.write_latexmkrc()
            shutil.copy(fiche.latexmkrcname, "latexmkrc")
        # LaTeX
        if 'tex' in options.format or 'pdf' in options.format:
            fiche.write_tex()
            for command in options.pipe:
                formatted = command.format(fiche.texname)
                if formatted == command:
                    formatted = '{} {}'.format(command, fiche.texname)
                subprocess.run(
                        formatted,
                        shell=True,
                        cwd=fiche.workingdir,
                        )
            if 'tex' in options.format:
                shutil.copy(fiche.texname, "{}.tex".format(options.output))
        if 'pdf' in options.format:
            fiche.write_pdf()
            shutil.copy(fiche.pdfname, "{}.pdf".format(options.output))

def do_ls(options): # pylint: disable=unused-argument
    """Perform the `ls` command."""
    bag = ExerciseBag().filter_tags(*options.tags).filter_desc(*options.desc)
    for name in sorted(bag, key=str.lower):
        if options.verbose:
            print(u"{}: {} {}".format(
                name,
                bag[name].description(),
                bag[name].tags,
                ))
        else:
            print(name)

def do_tags(options): # pylint: disable=unused-argument
    """Perform the `tags` command."""
    bag = ExerciseBag()
    tags = set().union(
        *(set(exo.tags) for exo in bag.values())
        )
    print("\n".join(sorted(tags, key=str.lower)))

COMMANDS = {
    "generate": do_generate,
    "ls": do_ls,
    "test": do_test,
    "dummy": do_dummy,
    "tags": do_tags,
    }

def main():
    """Main function"""
    options = argument_parser().parse_args(sys.argv[1:])

    try:
        COMMANDS[options.command](options)
    except PyromathsException as error:
        logging.error(error)
        sys.exit(1)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#


# #- additions / soustractions de vecteurs ( difficulté : ne pas sortir du cadre )
# #- multiplication d'un vecteur par un réel +  add + sous ( difficulté : ne pas sortir du cadre )
# #- lire les coordonnées d'un vecteur
# #- placer un point B connaissant A et les coordonnées de vec{AB} : avec coordonnées ou i et j ( difficulté : ne pas sortir du cadre )
# #- calcul de norme
# #- simplifier des sommes
# #- problèmes de colinéarité

from pyromaths.classes.Vecteurs import randvect, Vecteur
from pyromaths.classes.Racine import simplifie_racine
import math
from random import randint, shuffle
from pyromaths.ex import LegacyExercise

def dist_bords(a, b):
    '''Calcule les distances minimales d'un point de coordonnées (a,b) aux bords du cadre, selon l'axe x et y.'''
    x = min(a, 18 - a)  # la largeur vaut 18
    y = min(b, 10 - b)  # la hauteur vaut 10
    return (x, y)

def pair(n):
    '''Retourne le plus petit entier pair strictement plus grand que n'''
    if (n % 2 == 0):
        return n + 2
    else:
        return n + 1

def AffNom(u, crd=0):
    '''Renvoie les coordonnées pour l'affichage du nom du vecteur u.'''
    if u.x == 0 and math.fabs(u.y) > 2:
        coord = (0, u.y // 2)
    elif u.x == 0:
        coord = (-0.5, u.y // 2)
    elif u.y == 0 and math.fabs(u.x) > 2:
        coord = (u.x // 2, 0)
    elif u.y == 0:
        coord = (u.x // 2, -0.5)
    elif math.fabs(u.x) + math.fabs(u.y) < 3:
        coord = (u.x / 2 + 0.5, u.y / 2 + 0.5)
    else:
        coord = (u.x // 2, u.y // 2)
    return str(coord[0]) + "," + str(coord[1])

def ChoixVecteur(u, v, w, x, y):
    listecoeff = [0.5, -0.5, -1, -2, 2, 3, -3]
    listevect = [(u, "u"), (v, "v"), (w, "w")]
    shuffle(listecoeff)
    shuffle(listevect)
    for vec in listevect:
        for coeff in listecoeff:
            if (0 <= x + coeff * vec[0].x <= 18) and (0 <= y + coeff * vec[0].y <= 10):
                return (coeff, coeff * vec[0], vec[1])



def repr_somme(u, v, u1, u2, cor, larg=0):
    '''Représente la somme des vecteurs u + v.'''

    a = u + v

    if (u.x * a.x >= 0):
        largeur = max(math.fabs(u.x), math.fabs(a.x))
        if (a.x > 0):
            departx = 0
        elif (a.x == 0):
            departx = -u.x / 2 + math.fabs(u.x) / 2
        else:
            departx = max(math.fabs(u.x), math.fabs(a.x))
    else:
        largeur = math.fabs(u.x) + math.fabs(a.x)
        if (u.x >= 0):
            departx = -a.x
        else:
            departx = -u.x
    if (u.y * a.y >= 0):
        hauteur = max(math.fabs(u.y), math.fabs(a.y))
        if (a.y > 0):
            departy = 0
        elif (a.y == 0):
            departy = -u.y / 2 + math.fabs(u.y) / 2
        else:
            departy = max(math.fabs(u.y), math.fabs(a.y))
    else:
        hauteur = math.fabs(u.y) + math.fabs(a.y)
        if (u.y >= 0):
            departy = -a.y
        else:
            departy = -u.y

    if int(larg) + largeur > 18:
        cor.append("\\par")  # Figure trop large avec la précédente, il faut passer à une nouvelle ligne.

    depart = "(" + str(departx) + "," + str(departy) + ")"
    largeur = str(pair(int(largeur)))
    hauteur = str(pair(int(hauteur)))

    cor.append(u"\\begin{pspicture*}(0,0)(" + largeur + "," + hauteur + ")")
    cor.append(u"\\psgrid[subgriddiv=2, gridlabels=0pt]")
    cor.append(u"\\psset{unit=10mm,arrowscale=2}")

    cor.append(u"\\rput" + depart + "{")
    cor.append(u"\\psline[linewidth=1pt, linecolor=DarkGreen]{|->}(0, 0)(" + str(u.x) + ", " + str(u.y) + ")")  # # Premier Vecteur
    cor.append(u"\\rput(" + AffNom(u) + ") \
          {\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{\\textcolor{DarkGreen}{$\\overrightarrow{" + u1 + "}$}}}")

    cor.append(u"\\psline[linewidth=1pt, linecolor=DarkBlue]{|->}(" + str(u.x) + ", " + str(u.y) + ")(" + str(a.x) + ", " + str(a.y) + ")")  # # 2e Vecteur
    k = Vecteur(u.x + a.x, u.y + a.y)
    cor.append(u"\\rput(" + AffNom(k) + ") \
          {\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{\\textcolor{DarkBlue}{$\\overrightarrow{" + u2 + "}$}}}")
    if len(u2) > 1:
        sgn = "-"
    else:
        sgn = "+"
    cor.append(u"\\psline[linestyle=dashed, linewidth=1pt, linecolor=DarkRed]{|->}(0, 0)(" + str(a.x) + ", " + str(a.y) + ")")  # # Résultat de l'opération
    cor.append(u"\\rput(" + AffNom(a) + ") \
          {\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{\\textcolor{DarkRed}{$\\overrightarrow{" + u1 + "}" + sgn + "\\overrightarrow{" + u2[-1] + "}$}}}")

    cor.append(u"}")
    cor.append(u"\\end{pspicture*}")
    return largeur  # # récupérer la largeur pour éviter d'aligner des figures trop larges sur la feuille

def _vecteurs_add():
    '''Exercice sur la définition des vecteurs et leurs sommes.'''
    t = None
    while not t:
    # Pour être sûr que l'exercice ait des solutions
        (u, posux, posuy) = randvect(0, 10)
        (v, posvx, posvy) = randvect(math.fabs(u.x) + 1, 10)
        (w, poswx, poswy) = randvect(math.fabs(v.x) + math.fabs(u.x) + 2, 10)

        # # Construction du point pour la question 2
        if 18 - poswx - max(w.x, 0) > 0:
            restes = (18 - poswx - max(w.x, 0), 10)
            pointy = randint(0, 10)
        elif poswy + min(w.y, 0) > 10 - poswy - max(w.y, 0):
            restes = (poswx + min(w.x, 0), poswy + min(w.y, 0))
            pointy = randint(0, restes[1])
        else:
            restes = (poswx + min(w.x, 0), 10 - poswy - max(w.y, 0))
            pointy = randint(10 - restes[1], 10)

        pointx = randint(18 - restes[0], 18)

        t = ChoixVecteur(u, v, w, pointx, pointy)

    exo = ["\\exercice"]
    cor = ["\\exercice*"]

    exo.append(u"\\begin{pspicture*}(0,0)(18,10)")
    exo.append(u"\\psgrid[subgriddiv=2, gridlabels=0pt]")
    exo.append(u"\\psset{unit=10mm,arrowscale=2}")

    cor.append(u"\\begin{pspicture*}(0,0)(18,10)")
    cor.append(u"\\psgrid[subgriddiv=2, gridlabels=0pt]")
    cor.append(u"\\psset{unit=10mm,arrowscale=2}")

    exo.append(u"\\psdot(" + str(pointx) + "," + str(pointy) + ")")

    if pointx < 18 and pointy < 10:
        nompoint = str(pointx + 0.5) + "," + str(pointy + 0.5)
    else:
        nompoint = str(pointx - 0.5) + "," + str(pointy - 0.5)

    exo.append(u"\\rput(" + nompoint + "){\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$A$}}")

    cor.append(u"\\psdot(" + str(pointx) + "," + str(pointy) + ")")
    cor.append(u"\\rput(" + nompoint + "){\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$A$}}")

    cor.append(u"\psline[linecolor=DarkBlue]{|->}(" + str(pointx) + "," + str(pointy) + ")(" + str(pointx + t[1].x) + ", " + str(pointy + t[1].y) + ")")

    bx = pointx + t[1].x
    by = pointy + t[1].y

    if bx < 18 and by < 10:
        nompoint = str(bx + 0.5) + "," + str(by + 0.5)
    else:
        nompoint = str(bx - 0.5) + "," + str(by - 0.5)

    cor.append(u"\\psdot(" + str(pointx + t[1].x) + "," + str(pointy + t[1].y) + ")")
    cor.append(u"\\rput(" + nompoint + "){\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$B$}}")

    for vec in [(u, posux, posuy, "u"), (v, posvx, posvy, "v"), (w, poswx, poswy, "w")]:
        exo.append(u"\\rput(" + str(vec[1]) + "," + str(vec[2]) + "){")
        exo.append(u"\psline{|->}(0, 0)(" + str(vec[0].x) + ", " + str(vec[0].y) + ")")

        exo.append(u"\\rput(" + AffNom(vec[0]) + ") \
                     {\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$\\overrightarrow{" + vec[3] + "}$}}")
        exo.append(u"}")
    exo.append(u"\\end{pspicture*}")


    for vec in [(u, posux, posuy, "u"), (v, posvx, posvy, "v"), (w, poswx, poswy, "w")]:
        if vec[0].y > 0:
            plus = 1
        else:
            plus = 0
        cor.append(u"\\rput(" + str(vec[1]) + "," + str(vec[2]) + "){")
        cor.append(u"\\psline{|->}(0, 0)(" + str(vec[0].x) + ", " + str(vec[0].y) + ")")
        cor.append(u"\\psline[linestyle=dashed,linecolor=DarkRed](0, 0)(" + str(vec[0].x) + ", 0)(" + str(vec[0].x) + "," + str(vec[0].y) + ")")
        cor.append(u"\\rput(" + AffNom(vec[0]) + "){\\psframebox[linecolor=white, fillcolor=white, \
                     fillstyle=solid]{$\\overrightarrow{" + vec[3] + "}\\ (" + str(vec[0].x) + ";" + str(vec[0].y) + ")$}}")
        cor.append(u"}")
    cor.append(u"\\end{pspicture*}")

    exo.append("\\par")
    cor.append("\\par")
    exo.append(_(u"On se place dans un repère orthonormé et on considère les vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$ ci-dessous."))
    cor.append(_(u"On se place dans un repère orthonormé et on considère les vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$ ci-dessous."))

    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")

    exo.append(_(u"\\item Lire les coordonnées de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$."))
    cor.append(_(u"\\item Lire les coordonnées de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$."))

    cor.append("\\par")
    cor.append(_(u"Un petit rappel : l'abscisse d'un vecteur est la différence d'abscisse entre le fin et le début du vecteur. \
                 Concernant le vecteur $\\overrightarrow{u}$, son abscisse est $" + str(u.x) + u"$. \
                 On lit également son ordonnée : $") + str(u.x) + _(u"$. \
                 Donc les coordonnées de $\\overrightarrow{u}$ sont $(") + str(u.x) + ", " + str(u.y) + _(u" )$. \
                 Des pointillés ont été ajoutés sur la figure pour faciliter la lecture des coordonnées."))
    cor.append(_(u"De même, les coordonnées de $\\overrightarrow{v}$ sont $(") + str(v.x) + ", " + str(v.y) + _(u" )$ \
                 et les coordonnées de $\\overrightarrow{w}$ sont $(") + str(w.x) + ", " + str(w.y) + " )$.")

    exo.append(_(u"\\item Placer un point B de sorte que le vecteur $\\overrightarrow{AB}$ soit égal à $") + str(t[0]) + " \\times \\overrightarrow{" + t[2] + "}$.")
    cor.append(_(u"\\item Placer un point B de sorte que le vecteur $\\overrightarrow{AB}$ soit égal à $") + str(t[0]) + " \\times \\overrightarrow{" + t[2] + "}$.")

    cor.append(u"\\par")
    cor.append(_(u"Le plus simple pour répondre à cette question est de calculer les coordonnées du vecteur $") + str(t[0]) + " \\times \\overrightarrow{" + str(t[2]) + "}$.")
    cor.append(_(u"Cela se fait en multipliant les coordonnées de $\\overrightarrow{") + str(t[2]) + "}$ par $" + str(t[0]) + _(u"$, ce qui donne comme résultat $(") + str(t[1].x) + ";" + str(t[1].y) + ")$.")
    cor.append(_(u"En partant du point A et en respectant ces coordonnées, on dessine un vecteur (en bleu sur la figure ci-dessus) qui indique l'emplacement du point B."))

    exo.append(_(u"\\item Calculer les normes de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$."))
    cor.append(_(u"\\item Calculer les normes de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$."))

    if u.x ** 2 + u.y ** 2 == simplifie_racine(u.x ** 2 + u.y ** 2)[1]:  # Cas où la simplification est la même, donc inutile d'écrire deux fois la même chose.
        Norm_u = "$"
    else:
        Norm_u = "=" + str(u.normeTex()) + "$"

    if v.x ** 2 + v.y ** 2 == simplifie_racine(v.x ** 2 + v.y ** 2)[1]:
        Norm_v = "$"
    else:
        Norm_v = "=" + str(v.normeTex()) + "$"

    if w.x ** 2 + w.y ** 2 == simplifie_racine(w.x ** 2 + w.y ** 2)[1]:
        Norm_w = "$"
    else:
        Norm_w = "=" + str(w.normeTex()) + "$"

    cor.append("\\par")
    cor.append(u"$\|\\overrightarrow{u}\|=\\sqrt{(" + str(u.x) + ")^2+(" + str(u.y) + ")^2}=\\sqrt{" + str(u.x ** 2) + " + " + str(u.y ** 2) + "}= \
                 \\sqrt{" + str(u.x ** 2 + u.y ** 2) + "}" + Norm_u + ".\\par")
    cor.append(_(u"De la même manière, on obtient :"))

    cor.append(u"$\|\\overrightarrow{v}\|=\\sqrt{(" + str(v.x) + ")^2+(" + str(v.y) + ")^2}=\\sqrt{" + str(v.x ** 2) + " + " + str(v.y ** 2) + "}= \
                 \\sqrt{" + str(v.x ** 2 + v.y ** 2) + "}" + Norm_v + " et \\par")
    cor.append(u"$\|\\overrightarrow{w}\|=\\sqrt{(" + str(w.x) + ")^2+(" + str(w.y) + ")^2}=\\sqrt{" + str(w.x ** 2) + " + " + str(w.y ** 2) + "}= \
                 \\sqrt{" + str(w.x ** 2 + w.y ** 2) + "}" + Norm_w + ".\\par")

    exo.append(_(u"\\item Dessiner des représentants des vecteurs $\\overrightarrow{u}+\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{w}$ \
                 et $\\overrightarrow{v}+\\overrightarrow{w}$."))
    cor.append(_(u"\\item Dessiner des représentants des vecteurs $\\overrightarrow{u}+\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{w}$ \
                 et $\\overrightarrow{v}+\\overrightarrow{w}$."))

    cor.append("\\par")
    cor.append(_(u"Pour dessiner les sommes ou différences de vecteurs, il faut les mettre \"bouts à bouts\", \
                 comme sur les figures qui suivent :\\par"))
    i = repr_somme(u, v, 'u', 'v', cor)
    repr_somme(u, -v, 'u', '-v', cor, i)
    cor.append("\\par")
    i = repr_somme(u, -w, 'u', '-w', cor)
    repr_somme(v, w, 'v', 'w', cor, i)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")

    return exo, cor

class vecteurs_add(LegacyExercise):
    """Vecteurs"""

    tags = ["Seconde"]
    function = _vecteurs_add



# Copyright (C) 2016 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""Generic stuff for pyromaths command line interface"""

import argparse

class PyromathsException(Exception):
    """Generic exception for this module."""
    pass

def exercise_argument(string=""):
    """Return the exercises matching ``string``.

    :param str string: a string, option of one of `testexo` commands.
    :rtype: dict
    :return: A dictionary with exercises as keys, and sets of integers (seeds)
    as values.
    """
    splitted = string.split(":")
    if len(splitted) == 1:
        name = string
        seeds = []
    elif len(splitted) == 2:
        name, seeds = string.split(":")
        try:
            seeds = [int(seed) for seed in seeds.split(",")]
        except ValueError:
            raise argparse.ArgumentTypeError("Seeds must be a comma separated list of integers.")
    else:
        raise argparse.ArgumentTypeError("Seeds must be a comma separated list of integers.")

    return (name, seeds)




# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if notPopen, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

import random
from pyromaths.outils.Affichage import decimaux
from pyromaths.ex import LegacyExercise

def noms_sommets(nb):
    """Renvoie nb noms de sommets"""
    (listenb, listepts) = ([], [])
    for i in range(26):
        listenb.append(i + 65)
    for i in range(nb):
        listepts.append(str(chr(listenb.pop(random.randrange(26 - i)))))
    listepts.sort()
    return tuple(listepts)

def angle(coordo):
    """Angle pour placer le nom du point en fonction du quadrant"""
    l = []
    for coord in coordo:
        if coord[0] >= 0 and coord[1] >= 0:
            ang = '45'
        if coord[0] >= 0 and coord[1] < 0:
            ang = '-45'
        if coord[0] < 0 and coord[1] < 0:
            ang = '-135'
        if coord[0] < 0 and coord[1] >= 0:
            ang = '135'


        l.append(ang)
    return l

def place_points(coordo, points):
    """coordo: liste de coordonnées ;
    points : noms des points associés
    génère le code latex pour placer tous les points"""
    l = angle(coordo)
    text = "\\pstGeonode[PointSymbol=x,PosAngle={"
    for ang in l :
        text = text + ang + ','
    text = text + "},PointNameSep=0.4]"
    i = 0
    while i < len(coordo):
        text = text + str(coordo[i]) + "{" + str(points[i]) + "}"
        i = i + 1
    return text

def valide(coord, liste_coord):
    """Évite deux points trop proches l'un de l'autre"""
    rep = True
    for c in liste_coord:
        if abs(coord[0] - c[0]) <= 0.5 and abs(coord[1] - c[1]) <= 0.5:
            rep = False
            break
    return rep

def quadrant(coord, liste_coord):
    """Évite 2 points dans le même quadrant"""
    rep = True
    for c in liste_coord:
        if coord[0] * c[0] >= 0 and coord[1] * c[1] >= 0:
            rep = False
            break
    return rep


def coordo_pts(nb):
    """Génère une liste de nb coordonnées. Je m'arrange pour avoir des points
    sur les 4 quadrants et sur les deux axes"""
    j = 0
    k = 0
    l = []
    i = 0
    while i < 4 and i < nb:
        j = j + 1
        if j == 600:
            # Correction d'un bug
            l, i, j = [], 0, 0
        a = random.randrange(-9, 10) / 2
        b = random.randrange(-9, 10) / 2
        if ((a, b) not in l) and valide((a, b), l)and quadrant((a, b), l):
            l.append((a, b))
            i = i + 1
    if nb >= 4:
        a = random.randrange(-9, 10) / 2
        while not valide((0, a), l):
            a = random.randrange(-9, 10) / 2
        rg = random.randrange(0, len(l) - 1)
        l[rg:rg] = [(0, a)]
        i = i + 1

    if nb >= 5:
        b = random.randrange(-9, 10) / 2
        while not valide((b, 0), l):
            b = random.randrange(-9, 10) / 2
        rg = random.randrange(0, len(l) - 1)
        l[rg:rg] = [(b, 0)]
        i = i + 1


    while i >= 6 and i < nb and i < 10:
        a = random.randrange(-9, 10) / 2
        b = random.randrange(-9, 10) / 2
        j = j + 1
        if j == 600:
            break
        if ((a, b) not in l) and valide((a, b), l)and quadrant((a, b), l[6:i]):
            l.append((a, b))
            i = i + 1
    if nb >= 11 and len(l) == 10:
        a = random.randrange(-9, 10) / 2
        while not valide((0, a), l):
            a = random.randrange(-9, 10) / 2
        rg = random.randrange(6, len(l) - 1)
        l[rg:rg] = [(0, a)]
        i = i + 1
    if nb >= 11 and len(l) == 11:
        b = random.randrange(-9, 10) / 2
        while not valide((b, 0), l):
            b = random.randrange(-9, 10) / 2
        rg = random.randrange(6, len(l) - 1)
        l[rg:rg] = [(b, 0)]
        i = i + 1

    while i >= 12 and i < nb :
        a = random.randrange(-9, 10) / 2
        b = random.randrange(-9, 10) / 2
        k = k + 1
        if k == 600:
            break
        if ((a, b) not in l) and valide((a, b), l):
            l.append((a, b))
            i = i + 1
    return l


def affiche_coord(coord):
    """Affiche les coordonnées des points au format LaTeX"""
    return '\\hbox{$(' + decimaux(str(coord[0]), 1) + '~;~' + \
                                             decimaux(str(coord[1]), 1) + ')$}'

def tex_liste_co(liste_coord):
    """prend une liste de coordonnées et renvoie la liste en string prête pour latex
    #\hbox évite que les coordonnées soient coupées à l'affichage"""
    i = 0
    tlc = []
    while i < len(liste_coord):
        tlc.append(affiche_coord(liste_coord[i]))
        i = i + 1
    return tlc

def _reperage():
    nbpts = 13
    noms_pts = (noms_sommets(nbpts))
    coord_pts = coordo_pts(nbpts)
    voc = [_('abscisse'), _(u'ordonnée')]
    rg1 = random.randrange(0, 2)
    rg2 = abs(rg1 - 1)
    while len(coord_pts) < nbpts:
        coord_pts = coordo_pts(nbpts)
    exo = ["\\exercice",
         "\\parbox{0.4\\linewidth}{",
         "\\begin{enumerate}",
         _(u"\\item Donner les coordonnées des points %s, %s, %s, %s, %s et %s.") % tuple(noms_pts[0:6]),
         _(u"\\item Placer dans le repère les points %s, %s, %s, %s, %s et %s") % noms_pts[6:12] + _(u" de coordonnées respectives %s, %s, %s, %s, %s et %s. ") % tuple(tex_liste_co(coord_pts[6:12])),
         _(u"\\item Placer dans le repère le point %s d'%s %s et d'%s %s") % (noms_pts[12], voc[rg1], decimaux(str(coord_pts[12][rg1])), voc[rg2], decimaux(str(coord_pts[12][rg2]))),
         "\\end{enumerate}}\\hfill ",
         "\\parbox{0.55\\linewidth}{",
         "\\psset{unit=0.8cm}",
         "\\begin{pspicture}(-4.95,-4.95)(4.95,4.95)",
         "\\psgrid[subgriddiv=2, subgridcolor=lightgray, gridlabels=8pt](0,0)(-5,-5)(5,5)",
         "\\psline[linewidth=1.2pt]{->}(-5,0)(5,0)",
         "\\psline[linewidth=1.2pt]{->}(0,-5)(0,5)",
         place_points(coord_pts[0:6], noms_pts[0:6]),
         "\\end{pspicture}}"]
    cor = ["\\exercice*",
         "\\parbox{0.4\\linewidth}{",
         "\\psset{unit=0.8cm}",
         "\\begin{pspicture}(-5,-5)(5,5)",
         "\\psgrid[subgriddiv=2, subgridcolor=lightgray, gridlabels=8pt](0,0)(-5,-5)(5,5)",
         "\\psline[linewidth=1.2pt]{->}(-5,0)(5,0)",
         "\\psline[linewidth=1.2pt]{->}(0,-5)(0,5)",
         place_points(coord_pts[0:13], noms_pts[0:13]),
         "\\end{pspicture}}\\hfill",
         "\\parbox{0.5\\linewidth}{",
         "\\begin{enumerate}",
         _(u"\\item Donner les coordonnées des points %s, %s, %s, %s, %s et %s.") % tuple(noms_pts[0:6])]
    i = 0
    while i < 6:
        cor.append(_(u"Les coordonnées du point %s sont %s \n") % (noms_pts[i], affiche_coord(coord_pts[i])))
        i = i + 1
    cor[len(cor):len(cor)] = [_(u"\\item Placer dans le repère les points %s, %s, %s, %s, %s et %s") % noms_pts[6:12] +       _(u" de coordonnées respectives %s, %s, %s, %s, %s et %s. ") % tuple(tex_liste_co(coord_pts[6:12])),
         _(u"\\item Placer dans le repère le point %s d'%s %s et d'%s %s") % (noms_pts[12], voc[rg1], decimaux(str(coord_pts[12][rg1])), voc[rg2], decimaux(str(coord_pts[12][rg2]))),
         "\\end{enumerate}"]
    cor.append("}")
    return(exo, cor)

class reperage(LegacyExercise):
    """Repérage"""

    tags = ["Cinquième"]
    function = _reperage



# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if notPopen, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

import random
# from pyromaths.outils.Affichage import decimaux
from pyromaths.ex import LegacyExercise

def noms_sommets(nb):
    """Renvoie nb noms de sommets"""

    (listenb, listepts) = ([], [])
    for i in range(26):
        listenb.append(i + 65)
    for i in range(nb):
        listepts.append(str(chr(listenb.pop(random.randrange(26 - i)))))
    listepts.sort()
    return listepts

def connect(pt, F1, F2):
    """renvoie les sommets connectes à pt"""
    if pt in F1:
        result = [F1[F1.index(pt) - 1], F1[(F1.index(pt) + 1) % len(F1)], F2[F1.index(pt)]]
    else:
        result = [F2[F2.index(pt) - 1], F2[(F2.index(pt) + 1) % len(F2)], F1[F2.index(pt)]]
    return result


def reponse1(pt, F1, F2):
    """renvoie la réponse à la première question"""
    conn = connect(pt, F1, F2)
    ptc1 = conn[random.randrange(0, len(conn))]
    conn.remove(ptc1)
    ptc2 = conn[random.randrange(0, len(conn))]
    conn.remove(ptc2)
    ptc3 = conn[0]
    F3 = [F2[0], F1[0], F1[3], F2[3]]
    F4 = [F2[0], F2[1], F1[1], F1[0]]
    F5 = [F1[1], F2[1], F2[2], F1[2]]
    F6 = [F1[2], F2[2], F2[3], F1[3]]
    F2b = F2[:]
    F2b.reverse()
    if (pt in F1) and (ptc1 in F1) and (ptc2 in F1):
        i = F1.index(pt)
        j = F2.index(ptc3)
        res = [pt, F1[(i + 1) % 4], F1[(i + 2) % 4], F1[(i + 3) % 4], ptc3, F2[(j + 1) % 4], F2[(j + 2) % 4], F2[(j + 3) % 4]]
    elif (pt in F2) and (ptc1 in F2) and (ptc2 in F2):
        i = F2b.index(pt)
        j = F1.index(ptc3)
        res = [pt, F2b[(i + 1) % 4], F2b[(i + 2) % 4], F2b[(i + 3) % 4], ptc3, F1[(j - 1) % 4], F1[(j - 2) % 4], F1[(j - 3) % 4]]
    elif (pt in F3) and (ptc1 in F3) and (ptc2 in F3):
        i = F3.index(pt)
        j = F5.index(ptc3)
        res = [pt, F3[(i + 1) % 4], F3[(i + 2) % 4], F3[(i + 3) % 4], ptc3, F5[(j - 1) % 4], F5[(j - 2) % 4], F5[(j - 3) % 4]]
    elif (pt in F4) and (ptc1 in F4) and (ptc2 in F4):
        i = F4.index(pt)
        j = F6.index(ptc3)
        res = [pt, F4[(i + 1) % 4], F4[(i + 2) % 4], F4[(i + 3) % 4], ptc3, F6[(j - 1) % 4], F6[(j - 2) % 4], F6[(j - 3) % 4]]
    elif (pt in F5) and (ptc1 in F5) and (ptc2 in F5):
        i = F5.index(pt)
        j = F3.index(ptc3)
        res = [pt, F5[(i + 1) % 4], F5[(i + 2) % 4], F5[(i + 3) % 4], ptc3, F3[(j - 1) % 4], F3[(j - 2) % 4], F3[(j - 3) % 4]]
    elif (pt in F6) and (ptc1 in F6) and (ptc2 in F6):
        i = F6.index(pt)
        j = F4.index(ptc3)
        res = [pt, F6[(i + 1) % 4], F6[(i + 2) % 4], F6[(i + 3) % 4], ptc3, F5[(j - 1) % 4], F5[(j - 2) % 4], F5[(j - 3) % 4]]
    return res




def reponse2(pt1, ptc1, F1, F2):
    """renvoie la réponse à la deuxième question et le segment[p1ptc1]"""
    rep = []
    connects = connect(pt1, F1, F2)
    for pt in connects:
        if pt != ptc1:
            rep.append('[' + pt1 + pt + ']')
    connects2 = connect(ptc1, F1, F2)
    for pt in connects2:
        if pt != pt1:
            rep.append('[' + ptc1 + pt + ']')
    rep.append('[' + pt1 + ptc1 + ']')
    return tuple(rep)
def reponse3(pt2, ptc2, F1, F2):
    """renvoie la réponse à la troisièmequestion et le segment[pt2ptc2]"""
    if (pt2 in F1) and (ptc2 in F1):
        copy = F1[:]
        i = F1.index(pt2)
        j = F1.index(ptc2)
        copy.remove(pt2)
        copy.remove(ptc2)
        k = F1.index(copy[0])
        l = F1.index(copy[1])
        rep = ('[' + F2[i] + F2[j] + ']', '[' + F1[k] + F1[l] + ']', '[' + F2[k] + F2[l] + ']', '[' + pt2 + ptc2 + ']')
    elif (pt2 in F2) and (ptc2 in F2):
        copy = F2[:]
        i = F2.index(pt2)
        j = F2.index(ptc2)
        copy.remove(pt2)
        copy.remove(ptc2)
        k = F2.index(copy[0])
        l = F2.index(copy[1])
        rep = ('[' + F1[i] + F1[j] + ']', '[' + F1[k] + F1[l] + ']', '[' + F2[k] + F2[l] + ']', '[' + pt2 + ptc2 + ']')
    else:
        if pt2 in F1:
            i = F1.index(pt2)
        else:
            i = F2.index(pt2)
        rep = ('[' + F1[(i + 1) % len(F1)] + F2[(i + 1) % len(F1)] + ']', '[' + F1[(i + 2) % len(F1)] + F2[(i + 2) % len(F1)] + ']', '[' + F1[(i + 3) % len(F1)] + F2[(i + 3) % len(F1)] + ']', '[' + pt2 + ptc2 + ']')

    return tuple(rep)

def _espace():
    """Face 1, Face 2"""
    noms_pts = (noms_sommets(8))
    F1 = noms_pts[0:4]
    F2 = noms_pts[4:8]
    pt = noms_pts[random.randrange(0, len(noms_pts))]
    rp1 = reponse1(pt, F1, F2)
    pt1 = noms_pts[random.randrange(0, len(noms_pts))]
    conn1 = connect(pt1, F1, F2)
    pt2 = noms_pts[random.randrange(0, len(noms_pts))]
    conn2 = connect(pt2, F1, F2)
    ptc1 = conn1[random.randrange(0, len(conn1))]
    ptc2 = conn2[random.randrange(0, len(conn2))]
    rp2 = u"%s, %s, %s et %s sont les arêtes perpendiculaires à %s." % reponse2(pt1, ptc1, F1, F2)
    rp3 = u"%s, %s et %s sont les arêtes parallèles à %s." % reponse3(pt2, ptc2, F1, F2)
    exo = ["\\exercice",
         u"Les figures 1 et 2 représentent le même cube %s%s%s%s%s%s%s%s.\\\ " % tuple(noms_pts),
         "\\psset{xunit=1.0cm,yunit=1.0cm,dotstyle=*,dotsize=3pt 0,linewidth=0.8pt,arrowsize=3pt 2,arrowinset=0.25}",
         "\\begin{pspicture*}(-2,-0.38)(15,4.5)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](-1,3.5)(-0.5,4)",
         "\\rput[bl](-0.85,3.65){\\white{\\textbf{$1$}}}",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](3.5,1)(1.5,1)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(1.5,3)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(0.5,0)",
         "\\psframe[linecolor=Maroon](0.5,0)(2.5,2)",
         "\\psline[linecolor=Maroon](1.5,3)(3.5,3)",
         "\\psline[linecolor=Maroon](3.5,3)(3.5,1)",
         "\\psline[linecolor=Maroon](0.5,2)(1.5,3)",
         "\\psline[linecolor=Maroon](2.5,2)(3.5,3)",
         "\\psline[linecolor=Maroon](2.5,0)(3.5,1)",
         "\\rput[bl](0.22,2.08){%s}" % F1[0],
         "\\rput[bl](2.28,2.16){%s}" % F1[1],
         "\\rput[bl](2.6,-0.22){%s}" % F1[2],
         "\\rput[bl](0.15,-0.22){%s}" % F1[3],
         "\\rput[bl](1.4,3.08){%s}" % F2[0],
         "\\rput[bl](3.58,3.04){%s}" % F2[1],
         "\\rput[bl](3.58,1.04){%s}" % F2[2],
         "\\rput[bl](1.58,1.04){%s}" % F2[3],
         "\\psline[linecolor=Maroon](5.2,-04)(5.2,4)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](5.7,3.5)(6.2,4)",
         "\\rput[bl](5.85,3.65){\\white{\\textbf{$2$}}}",
         "\\psline[linecolor=Maroon](8.48,2.71)(7.18,2.42)",
         "\\psline[linecolor=Maroon](7.18,2.42)(7.18,0.46)",
         "\\psline[linestyle=dashed, linecolor=Maroon, dash=4pt 4pt](7.18,0.46)(8.48,0.75)",
         "\\psline[linestyle=dashed, linecolor=Maroon, dash=4pt 4pt](8.48,0.75)(8.48,2.71)",
         "\psline[linecolor=Maroon](10,2.46)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,2.17)(8.7,0.21)",
         "\\psline[linecolor=Maroon](8.7,0.21)(10,0.5)",
         "\\psline[linecolor=Maroon](10,0.5)(10,2.46)",
         "\\psline[linecolor=Maroon](10,2.46)(8.48,2.71)",
         "\\psline[linecolor=Maroon](7.18,2.42)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,0.21)(7.18,0.46)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](10,0.5)(8.48,0.75)",
         "\\rput[bl](8.68,2.27){%s}" % rp1[0],
         "\\rput[bl](10.06,2.5){%s}" % rp1[1],
         "\\rput[bl](8.71,-0.1){%s}" % rp1[3],
         "\\end{pspicture*}",
         "\\begin{enumerate}",
         u"\\item Compléter les sommets manquants de la figure 2.",
         u"\\item Donner toutes les arêtes perpendiculaires à [%s%s]." % (pt1, ptc1),
         u"\\item Donner toutes les arêtes parallèles à [%s%s]." % (pt2, ptc2),
         "\\end{enumerate}"]
    cor = ["\\exercice*",
         u"Les figures 1 et 2 représentent le même cube %s%s%s%s%s%s%s%s.\\\ " % tuple(noms_pts),
         "\\psset{xunit=1.0cm,yunit=1.0cm,dotstyle=*,dotsize=3pt 0,linewidth=0.8pt,arrowsize=3pt 2,arrowinset=0.25}",
         "\\begin{pspicture*}(-2,-0.38)(15,4.5)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](-1,3.5)(-0.5,4)",
         "\\rput[bl](-0.85,3.65){\\white{\\textbf{$1$}}}",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](3.5,1)(1.5,1)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(1.5,3)",
         "\\psline[linestyle=dashed,linecolor=Maroon,dash=4pt 4pt](1.5,1)(0.5,0)",
         "\\psframe[linecolor=Maroon](0.5,0)(2.5,2)",
         "\\psline[linecolor=Maroon](1.5,3)(3.5,3)",
         "\\psline[linecolor=Maroon](3.5,3)(3.5,1)",
         "\\psline[linecolor=Maroon](0.5,2)(1.5,3)",
         "\\psline[linecolor=Maroon](2.5,2)(3.5,3)",
         "\\psline[linecolor=Maroon](2.5,0)(3.5,1)",
         "\\rput[bl](0.22,2.08){%s}" % F1[0],
         "\\rput[bl](2.28,2.16){%s}" % F1[1],
         "\\rput[bl](2.6,-0.22){%s}" % F1[2],
         "\\rput[bl](0.15,-0.22){%s}" % F1[3],
         "\\rput[bl](1.4,3.08){%s}" % F2[0],
         "\\rput[bl](3.58,3.04){%s}" % F2[1],
         "\\rput[bl](3.58,1.04){%s}" % F2[2],
         "\\rput[bl](1.58,1.04){%s}" % F2[3],
         "\\psline[linecolor=Maroon](5.2,-04)(5.2,4)",
         "\\psframe[fillstyle=solid,fillcolor=darkgray,framearc=0.2](5.7,3.5)(6.2,4)",
         "\\rput[bl](5.85,3.65){\\white{\\textbf{$2$}}}",
         "\\psline[linecolor=Maroon](8.48,2.71)(7.18,2.42)",
         "\\psline[linecolor=Maroon](7.18,2.42)(7.18,0.46)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](7.18,0.46)(8.48,0.75)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](8.48,0.75)(8.48,2.71)",
         "\psline[linecolor=Maroon](10,2.46)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,2.17)(8.7,0.21)",
         "\\psline[linecolor=Maroon](8.7,0.21)(10,0.5)",
         "\\psline[linecolor=Maroon](10,0.5)(10,2.46)",
         "\\psline[linecolor=Maroon](10,2.46)(8.48,2.71)",
         "\\psline[linecolor=Maroon](7.18,2.42)(8.7,2.17)",
         "\\psline[linecolor=Maroon](8.7,0.21)(7.18,0.46)",
         "\\psline[linestyle=dashed,dash=4pt 4pt, linecolor=Maroon](10,0.5)(8.48,0.75)",
         "\\rput[bl](10.05,0.2){%s}" % rp1[2],
         "\\rput[bl](8.71,-0.1){%s}" % rp1[3],
         "\\rput[bl](8.1,0.79){%s}" % rp1[6],
         "\\rput[bl](6.8,0.2){%s}" % rp1[7],
         "\\rput[bl](10.06,2.5){%s}" % rp1[1],
         "\\rput[bl](8.68,2.27){%s}" % rp1[0],
         "\\rput[bl](7.08,2.54){%s}" % rp1[4],
         "\\rput[bl](8.5,2.83){%s}" % rp1[5],
         "\\end{pspicture*}",
         "\\begin{enumerate}",
         u"\\item Compléter les sommets manquants de la figure 2.",
         u"\\item Donner toutes les arêtes perpendiculaires à [%s%s].\\par " % (pt1, ptc1),
         rp2,
         u"\\item Donner toutes les arêtes parallèles à [%s%s]. \\par " % (pt2, ptc2),
         rp3,
         "\\end{enumerate}"]
    return (exo, cor)

class espace(LegacyExercise):
    """Représentation dans l\'espace"""

    tags = ["Sixième"]
    function = _espace



#!/usr/bin/env python3

"""Installateur"""

from setuptools import setup, find_packages
import codecs
import os


def readme():
    """Lecture du README"""
    with codecs.open("README.md", encoding="utf8", errors="replace") as file:
        return file.read()


# Chargement des variables VERSION, COPYRIGHT_YEAR
# Ceci n'est pas fait par un `import pyromaths.version` pour ne pas importer
# des dependances qui ne sont pas encore installees.
with codecs.open("pyromaths/version.py", encoding="utf8", errors="replace") as file:
    exec(compile(file.read(), "version.py", "exec"))

setup(
    name="pyromaths",
    version=VERSION,
    packages=find_packages(exclude=["tests*"]),
    install_requires=["jinja2"],
    include_package_data=True,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description="Create maths exercises in LaTeX and PDF format",
    url="http://www.pyromaths.org",
    download_url="http://www.pyromaths.org/telecharger/",
    license="GPLv3",
    entry_points={"console_scripts": ["pyromaths = pyromaths.__main__:main"]},
    keywords="exercices math latex school",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Education",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Code Generators",
        "Topic :: Text Processing :: Markup :: LaTeX",
    ],
    long_description=readme(),
    long_description_content_type="text/markdown",
    zip_safe=False,
    project_urls={
        "Documentation": "http://pyromaths.readthedocs.org",
        "Download": "https://www.pyromaths.org/telecharger/",
        "Forum": "https://forum.pyromaths.org",
        "Sources": "https://framagit.org/pyromaths/pyromaths",
        "Tickets": "https://framagit.org/pyromaths/pyromaths/issues",
        "Version en ligne": "https://enligne.pyromaths.org/",
    },
    #namespace_packages=['pyromaths'],
)



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import re

def suppr0(nombre):
    """Supprime le zéro inutile après la virgule d'un float, si c'est possible."""
    if round(nombre, 0) == nombre:
        return int(nombre)
    else:
        return nombre

#---------------------------------------------------------------------
# Affichages des nombres décimaux
#---------------------------------------------------------------------
def decimaux(nb, mathenv=0):
    """nb est un float ou une str"""
    pattern = re.compile(r"^(-?\d+)\.*(\d*)e?([\+\-]?\d*)$")
    entiere, decimale, exposant = pattern.search(format(float(nb), ".14g")).groups() #arrondit les floats
    if exposant:
        if int(exposant) > 0:
            if int(exposant) < len(decimale):
                entiere = entiere + decimale[:int(exposant)]
                decimale = decimale[int(exposant):]
            else:
                entiere = entiere + decimale + "0"*(int(exposant) - len(decimale))
                decimale = ''
        else:
            if -int(exposant) < len(entiere):
                decimale = entiere[len(entiere) + int(exposant):] + decimale
                entiere = entiere[:len(entiere) + int(exposant)]
            else:
                decimale = "0"*(-int(exposant) - len(entiere)) + str(abs(int(entiere))) + decimale.rstrip("0")
                if entiere[0] == "-": entiere = "-0"
                else: entiere = "0"

    pattern = re.compile(r"^(-?\d{1,3}?)" + "(\d{3})" * \
                         ((len(entiere) - 1 - (entiere[0] == '-')) // 3) + "$")
    partie_entiere = pattern.search(entiere).groups()
    if decimale and int(decimale):
        """Vérifie si la partie décimale existe et si elle est différente de zéro"""
        pattern = re.compile(r"^" + "(\d{3})" * ((len(decimale) - 1) // 3) + "(\d{1,3})?$")
        partie_decimale = pattern.search(decimale).groups()
        if mathenv:
            return "{,}".join(("\,".join(partie_entiere), "\,".join(partie_decimale)))
        else:
            return ",".join(("\,".join(partie_entiere), "\,".join(partie_decimale)))
    else:
        return "\,".join(partie_entiere)



# Pyromaths
#
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
#
# Copyright (C) 2018 -- Louis Paternault (spalax@gresille.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

"""Outils à utiliser avec les exercices utilisant Jinja2.
"""
import math
import numbers

def facteur(nombre, court="",
            parentheses=False,
            zero=False,
            arrondi=None,
            signe=False,
            operation=False,
            produit=False,
            variable="",
           ):
    r"""Renvoit le code :math:`LaTeX` correspondant au nombre, dans un ``\\numprint{}``, en gérant de nombreux cas particuliers.

    Les cas d'utilisation de cette fonction sont détaillés dans le :ref:`tutoriel pour écrire un nouvel exercice <ecrire-facteur>`.

    En utilisant les arguments, cette fonction permet de prendre en compte les arrondis, et de nombreux cas particuliers (dans chaque cas, c'est le 3 qui est destiné à être rendu par cette fonction.

    - ajout de parenthèses uniquement si nécessaire (produit `2×3` et `2×(-3)` plutôt que `2×-3` ou `2×(3)`) ;
    - ajout du signe, même s'il est positif (produit `1+3x` et `1-3x` plutôt que `1 3x` ou `1+-3x`) ;
    - n'affiche pas le nombre 1, et seulement le signe de -1 (produit `x` et `-x` plutôt que `1x` et `-1x`) ;
    - n'affiche pas l'opérande nulle d'un produit (produit `2` plutôt que `2+0x`) ;
    - etc.

    Le séparateur décimal est le point, mais celui-ci sera converti en une virgule par ``\\numprint{}``.

    :param float nombre: Nombre à formatter.
    :param str court: Version courte des arguments (voir plus loin).
    :param boolean parentheses: Encadre le nombre avec des parenthèses, s'il est négatif (utile pour produire `2×(-3)` plutôt que `2×3`).
    :param int arrondi: Arrondi le nombre. L'argument est le nombre de chiffres après la virgule à utiliser. Mettre cet argument à ``None`` pour ne pas arrondir.
    :param boolean zero: Ajoute des zéros à la fin du nombre pour avoir autant de chiffres que défini par `arrondi` (par exemple, avec cette option et un arrondi au centième, `2` devient `2.00`).
    :param boolean signe: Produit le signe, même si le nombre est positif (produit alors un +).
    :param boolean operation: Indique que le signe plus ou moins est une opération (comme `2-3`) et non pas un opérateur unaire (comme `2×(-3)`). Cela a pour effet d'espacer un peu le signe du nombre.
    :param boolean produit: Le facteur est une opérande d'un produit : le nombre 1 ne doit pas être affiché ; du nombre -1, on ne conserve que le signe ; rien n'est affiché si le nombre est 0 (pour produire `x`, `-x` ou rien plutôt que `1x`, `-1x`, `0x`).
    :param str variable: Seconde opérande du produit. Doit être du code :math:`LaTeX` valide.
    :return: Le nombre, formatté comme du code :math:`LaTeX`.
    :rtype: str

    :Dépendances entre arguments:

    - Le comportement n'est pas défini si les arguments ``operation`` et ``parentheses`` sont utilisées en même temps.

    :Version courte des arguments:

    Pour que les arguments soient moins verbeux, il est possible d'utiliser la version courte des arguments. Par exemple, ``facteur(2, produit=True, parentheses=True, arrondi=3)`` peut s'écrire comme ``facteur(2, '*p3')``.

    Cet argument est une chaîne de caractères, chaque caractère « activant » une option longue. Par exemple ``*p3`` active les options ``produit``, ``parentheses`` et ``arrondi=3``. L'ordre des caractères n'a aucune importance. Plusieurs comportement sont non-définis :

    - présence de caractères ne correspondant à aucun argument ;
    - utilisation conjointe des arguments courts et longs ;
    - utilisation de plusieurs chiffres (pour l'argument ``arrondi``).

    Les arguments sont :

    - ``p``: ``parentheses`` ;
    - ``z``: ``zero`` ;
    - ``s``: ``signe`` ;
    - ``o``: ``operation`` ;
    - ``*``: ``produit`` ;
    - chiffres ``0`` à ``9``: ``arrondi`` à l'entier correspondant ;
    - ``x``, ``X``, ``y``, ``Y``, ``n``, ``N`` : Raccourcis pour (respectivement) ``variable="x"``, ``variable="x^2"``, ``variable="y"``, ``variable="y^2"``, ``variable="n"``, ``variable="n^2"``.

    :Exemples:

    Cas de base
        >>> from pyromaths.outils.jinja2utils import facteur
        >>> facteur(2)
        \numprint{2}
        >>> facteur(2.0)
        \numprint{2}
        >>> facteur(2.3)
        \numprint{2.3}
        >>> facteur(-122.0)
        \numprint{-122}

    Arrondi
        >>> facteur(12345.6789, arrondi=None)
        \numprint{12345.6789}
        >>> facteur(12345.6789, arrondi=0)
        \numprint{12346}
        >>> facteur(12345.6789, arrondi=2)
        \numprint{12345.68}
        >>> facteur(.6789, arrondi=0)
        \numprint{1}
        >>> facteur(.6789, arrondi=2)
        \numprint{0.68}

    Zéro
        >>> facteur(12345.6789, arrondi=None, zero=True)
        \numprint{12345.6789}
        >>> facteur(12345, arrondi=2, zero=True)
        \numprint{12345.00}
        >>> facteur(12345, arrondi=2, zero=False)
        \numprint{12345}
        >>> facteur(12345.7, arrondi=2, zero=True)
        \numprint{12345.70}
        >>> facteur(12345.7, arrondi=2, zero=False)
        \numprint{12345.7}

    Parenthèse
        >>> facteur(-2, parentheses=True)
        \left(\numprint{-2}\right)
        >>> facteur(2, parentheses=True)
        \numprint{2}

    Signe
        >>> facteur(-2, signe=True)
        \numprint{-2}
        >>> facteur(2, signe=True)
        \numprint{+2}
        >>> facteur(2, signe=False)
        \numprint{2}

    Opération
        >>> facteur(-2, operation=False)
        \numprint{-2}
        >>> facteur(2, signe=True, operation=False)
        \numprint{+2}
        >>> facteur(-2, operation=True)
        -\numprint{2}
        >>> facteur(2, signe=True, operation=True)
        +\numprint{2}

    Produit
        >>> facteur(1, produit=True, variable="x")
        x
        >>> facteur(-1, produit=True, variable="x")
        -x
        >>> facteur(0, produit=True, variable="x")
        <BLANKLINE>
        >>> facteur(1, produit=False, variable="x")
        \numprint{1}\,x

    Variable
        >>> facteur(2, variable="x")
        \numprint{2}\,x
        >>> facteur(-1, produit=True, variable="x")
        -x

    Version courte des arguments
        >>> facteur(-2, court="2zXo")
        -\numprint{2.00}\,x^2
        >>> facteur(-2, court="2zXp")
        \left(\numprint{-2.00}\,x^2\right)
        >>> facteur(-2, court="2zY")
        \numprint{-2.00}\,y^2
        >>> facteur(-1, court="y*")
        -y
        >>> facteur(1, court="p*x")
        x
        >>> facteur(-1, court="p*x")
        \left(-x\right)
        >>> facteur(-2, court="p*x")
        \left(\numprint{-2}\,x\right)
        >>> facteur(2, court="p*x")
        \numprint{2}\,x
        >>> facteur(1, court="s*x")
        +x

    """

    # Format court des arguments
    if "p" in court:
        parentheses = True
    if "z" in court:
        zero = True
    if "s" in court:
        signe = True
    if "o" in court:
        operation = True
    if "*" in court:
        produit = True
    if "x" in court:
        variable = "x"
    if "y" in court:
        variable = "y"
    if "n" in court:
        variable = "n"
    if "X" in court:
        variable = "x^2"
    if "Y" in court:
        variable = "y^2"
    if "N" in court:
        variable = "n^2"
    for entier in range(10):
        if str(entier) in court:
            arrondi = entier

    # Cas particuliers : produit
    if produit:
        if nombre == 1:
            if operation or signe:
                return "+{}".format(variable)
            return variable
        if nombre == -1:
            if operation:
                return "-{}".format(variable)
            if parentheses:
                return r"\left(-{}\right)".format(variable)
            return "-{}".format(variable)
        if nombre == 0:
            return ""

    # Division du nombre en signe et valeur absolue
    absolu = abs(nombre)
    if signe and nombre >= 0:
        plusoumoins = '+'
    elif nombre < 0:
        plusoumoins = '-'
    else:
        plusoumoins = ''

    # Conversion du nombre en prenant compte de l'arrondi et des zéros
    # Je n'ai pas réussi à faire ça uniquement à coup de
    # str.format(). Je serai ravi si quelqu'un me prouvait que
    # c'est possible !
    if arrondi is None:
        if math.trunc(absolu) == absolu:
            strabsolu = str(int(absolu))
        else:
            strabsolu = str(absolu)
    elif arrondi == 0:
        strabsolu = "{:g}".format(round(absolu))
    else:
        entier = math.trunc(absolu)
        decimal = str(round(absolu - entier, arrondi))[2:]
        if zero:
            strabsolu = "{}.{:0<{arrondi}}".format(entier, decimal, arrondi=arrondi)
        elif entier == absolu:
            strabsolu = str(entier)
        else:
            strabsolu = "{}.{}".format(entier, decimal)

    # Ajout éventuel d'une espace devant la variable
    if variable:
        variable = r"\," + variable

    # Ajout éventuel des parenthèses
    if parentheses and nombre < 0:
        formatter = r"\left(\numprint{{{signe}{absolu}}}{variable}\right)"
    elif operation:
        formatter = r"{signe}\numprint{{{absolu}}}{variable}"
    else:
        formatter = r"\numprint{{{signe}{absolu}}}{variable}"

    # Enfin !
    return formatter.format(
        signe=plusoumoins,
        absolu=strabsolu,
        variable=variable,
        )

def iter_facteurs(liste, short):
    """Applique :func:`facteur` à une liste de nombres.

    - Les éléments peuvent être des nombres, ou pas (auquel cas la fonction :func:`str` est appliquée).
    - L'argument ``short`` est transmis à :func:`facteur`.
    """
    for element in liste:
        if isinstance(element, numbers.Number):
            yield facteur(element, short)
        else:
            yield str(element)

def matrice(listes, short=""):
    r"""Renvoit le code LaTeX d'une matrice.

    - La matrice est sous la forme d'une liste de lignes (où chaque ligne est une liste de coefficients).
    - L'argument ``short`` est transmis à :func:`facteur`.

    >>> from pyromaths.outils.jinja2utils import matrice
    >>> matrice([[1, 2], [3, 4]])
    \begin{pmatrix}\numprint{1} & \numprint{2}\\\numprint{3} & \numprint{4}\\\end{pmatrix}
    """
    text = u""
    text += r"\begin{pmatrix}"
    for ligne in listes:
        text += " & ".join(iter_facteurs(ligne, short))
        text += r"\\"
    text += r"\end{pmatrix}"

    return text



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from pyromaths.outils import Arithmetique
import random
from pyromaths.outils.Affichage import tex_coef
from pyromaths.ex import LegacyExercise

# carres=[2,3,5,6,7,10,11,13,15,17,19]

carres = [2, 3, 5, 6, 7, 10]  # ,11,13,15,17,19]


def exoaRb0(exo, cor, v):
    a = (tex_coef(v[0], '\\sqrt{%s}' % (v[6] * v[3] ** 2)), tex_coef(v[1],
         '\\sqrt{%s}' % (v[6] * v[4] ** 2), bplus=1), tex_coef(v[2],
         '\\sqrt{%s}' % (v[6] * v[5] ** 2), bplus=1))
    exo.append(u'\\[ \\thenocalcul = ' + '%s%s%s' % a + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + '%s%s%s' % a + '\\] ')
    a = (tex_coef(v[0], '\\sqrt{%s}' % v[3] ** 2), v[6], tex_coef(v[1],
         '\\sqrt{%s}' % v[4] ** 2, bplus=1), v[6], tex_coef(v[2],
         '\\sqrt{%s}' % v[5] ** 2, bplus=1), v[6])
    cor.append(u'\\[ \\thenocalcul = ' +
                     '%s\\times\\sqrt{%s}%s\\times\\sqrt{%s}%s\\times\\sqrt{%s}' %
                     a + '\\] ')
    a = (
        tex_coef(v[0], ''),
        v[3],
        v[6],
        tex_coef(v[1], '', bplus=1),
        v[4],
        v[6],
        tex_coef(v[2], '', bplus=1),
        v[5],
        v[6],
        )
    cor.append(u'\\[ \\thenocalcul = ' +
                     '%s\\times%s\\times\\sqrt{%s}%s\\times%s\\times\\sqrt{%s}%s\\times%s\\times\\sqrt{%s}' %
                     a + '\\] ')
    a = (tex_coef(v[0] * v[3], '\\sqrt{%s}' % v[6]), tex_coef(v[1] * v[4],
         '\\sqrt{%s}' % v[6], bplus=1), tex_coef(v[2] * v[5],
         '\\sqrt{%s}' % v[6], bplus=1))
    cor.append(u'\\[ \\thenocalcul = ' + '%s%s%s' % a + '\\] ')
    del a
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + '%s' % tex_coef(v[0] * v[3] + v[1] * v[4] + v[2] *
                     v[5], '\\sqrt{%s}' % v[6]) + '} \\] ')


def valeurs_aRb0(pyromax):  # renvoie (coef0, coef1, coef2, carre0, carre1, carre2, b)
    a = carres
    l = [Arithmetique.valeur_alea(-pyromax, pyromax) for dummy in range(3)]
    while True:
        t = [random.randrange(2, pyromax) for dummy in range(3)]
        if pyromax < 4 or t[0] != t[1] and t[0] != t[2] and t[1] != t[2]:
            break
    l.extend(t)
    l.append(a[random.randrange(len(a))])
    return tuple(l)


def exoaRb1(exo, cor, v):
    a = (v[3] * v[0] ** 2, v[3] * v[1] ** 2, v[3] * v[2] ** 2)
    exo.append(u'\\[ \\thenocalcul = ' + '\\sqrt{%s}\\times\\sqrt{%s}\\times\\sqrt{%s}' %
                     a + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + '\\sqrt{%s}\\times\\sqrt{%s}\\times\\sqrt{%s}' %
                     a + '\\] ')
    a = (v[0] ** 2, v[3], v[1] ** 2, v[3], v[2] ** 2, v[3])
    cor.append(u'\\[ \\thenocalcul = ' +
                     '\\sqrt{%s}\\times\\sqrt{%s}\\times\\sqrt{%s}\\times\\sqrt{%s}\\times\\sqrt{%s}\\times\\sqrt{%s}' %
                     a + '\\] ')
    a = (v[0], v[3], v[1], v[3], v[2], v[3])
    cor.append(u'\\[ \\thenocalcul = ' +
                     '%s\\times\\sqrt{%s}\\times%s\\times\\sqrt{%s}\\times%s\\times\\sqrt{%s}' %
                     a + '\\] ')
    a = ((v[0] * v[1]) * v[2], v[3], v[3])
    cor.append(u'\\[ \\thenocalcul = ' +
                     '%s\\times\\left(\\sqrt{%s}\\right)^2\\times\\sqrt{%s}' %
                     a + '\\] ')
    a = ((v[0] * v[1]) * v[2], v[3], v[3])
    cor.append(u'\\[ \\thenocalcul = ' + '%s\\times%s\\times\\sqrt{%s}' % a + '\\] ')
    del a
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + '%s' % tex_coef(((v[0] * v[1]) * v[2]) * v[3],
                     '\\sqrt{%s}' % v[3]) + '} \\] ')


def valeurs_aRb1(pyromax):  # renvoie (coef0, coef1, coef2, carre0, carre1, carre2, b)
    a = carres
    while True:
        l = [random.randrange(2, pyromax) for dummy in range(3)]
        if pyromax < 4 or l[0] != l[1] and l[0] != l[2] and l[1] != l[2]:
            break
    l.append(a[random.randrange(len(a))])
    return tuple(l)


def valeurs_aPbRc(pyromax):  # renvoie (coef0, coef1, coef2, carre0, carre1, carre2, b)
    while True:
        a = carres[random.randrange(len(carres))]
        b = carres[random.randrange(len(carres))]
        if a != b and Arithmetique.pgcd(a, b) != min(a, b):
            break
    while True:
        c = random.randrange(2, pyromax)
        d = Arithmetique.valeur_alea(-pyromax, pyromax)
        if c != d:
            break
    return (c, a, d, b)


def exo_aPbRc(exo, cor, v):
    a = (tex_coef(v[0], '\\sqrt{%s}' % v[1]), tex_coef(v[2],
         '\\sqrt{%s}' % v[3], bplus=1))
    exo.append(u'\\[ \\thenocalcul = ' + '\\left( %s%s \\right)^2' % a + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + '\\left( %s%s \\right)^2' % a + '\\] ')
    if v[2] > 0:
        sgn = '+'
    else:
        sgn = '-'
    a = (tex_coef(v[0], '\\sqrt{%s}' % v[1], bpc=1), sgn, tex_coef(v[0],
         '\\sqrt{%s}' % v[1]), tex_coef(abs(v[2]), '\\sqrt{%s}' % v[3]),
         tex_coef(abs(v[2]), '\\sqrt{%s}' % v[3], bpc=1))
    cor.append(u'\\[ \\thenocalcul = ' + '%s^2%s2\\times%s\\times%s+%s^2' % a + '\\] ')
    a = (v[0] ** 2, v[1], tex_coef((2 * v[0]) * v[2], '\\sqrt{%s}' % (v[1] *
         v[3]), bplus=1), v[2] ** 2, v[3])
    cor.append(u'\\[ \\thenocalcul = ' + '%s\\times %s %s+%s\\times %s' % a + '\\] ')
    a = (v[0] ** 2 * v[1] + v[2] ** 2 * v[3], tex_coef((2 * v[0]) * v[2],
         '\\sqrt{%s}' % (v[1] * v[3]), bplus=1))
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + '%s%s' % a + '} \\] ')


def valeurs_entier0(pyromax):  # renvoie (coef0, coef1, coef2, carre0, carre1, carre2, b)
    a = carres[random.randrange(len(carres))]
    while True:
        b = random.randrange(2, pyromax)
        c = Arithmetique.valeur_alea(-pyromax, pyromax)
        if b != c and abs(c) != 1:
            break
    return (b, c, a)


def exo_entier0(exo, cor, v):
    a = (v[0], tex_coef(v[1], '\\sqrt{%s}' % v[2], bplus=1), v[0],
         tex_coef(-v[1], '\\sqrt{%s}' % v[2], bplus=1))
    exo.append(u'\\[ \\thenocalcul = ' + '\\left( %s%s \\right)\\left( %s%s \\right)' %
                     a + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + '\\left( %s%s \\right)\\left( %s%s \\right)' %
                     a + '\\] ')
    a = (tex_coef(v[0], '', bpc=1), tex_coef(abs(v[1]), '\\sqrt{%s}' % v[2],
         bpc=1))
    cor.append(u'\\[ \\thenocalcul = ' + '%s^2-%s^2' % a + '\\] ')
    a = (v[0] ** 2, v[1] ** 2, v[2])
    cor.append(u'\\[ \\thenocalcul = ' + '%s-%s\\times %s' % a + '\\] ')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + '%s' % (v[0] ** 2 - v[1] ** 2 * v[2]) + '} \\] ')


def valeurs_entier1(pyromax):  # renvoie (coef0, coef1, coef2, carre0, carre1, carre2, b)
    a = carres[random.randrange(len(carres))]
    while True:
        v0 = random.randrange(2, pyromax)
        v1 = random.randrange(2, pyromax)
        b0 = random.randrange(2, pyromax)
        b1 = random.randrange(2, pyromax)
        a0 = (b1 * v0) * v1
        a1 = v1 * b0
        if b0 < b1 and Arithmetique.pgcd(a0, a1) != a1:
            break
    return (a0, b0, a1, b1, a)


def exo_entier1(exo, cor, v):
    a = (tex_coef(v[0], '\\sqrt{%s}' % (v[1] ** 2 * v[4])), tex_coef(v[2],
         '\\sqrt{%s}' % (v[3] ** 2 * v[4])))
    exo.append(u'\\[ \\thenocalcul = ' + '\\frac{%s}{%s}' % a + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + '\\frac{%s}{%s}' % a + '\\] ')
    a = (v[0], v[1] ** 2, v[4], v[2], v[3] ** 2, v[4])
    cor.append(u'\\[ \\thenocalcul = ' +
                     '\\frac{%s\\times\\sqrt{%s}\\times\\cancel{\\sqrt{%s}}}{%s\\times\\sqrt{%s}\\times\\cancel{\\sqrt{%s}}}' %
                     a + '\\] ')
    cor.append(u'\\[ \\thenocalcul = ' + '\\frac{%s\\times %s}{%s\\times %s}' % v[0:4] + '\\] ')
    cor.append(u'\\[ \\boxed{\\thenocalcul = ' + '%s' % (((v[0] * v[1]) // v[2]) // v[3]) + '} \\] ')

def _tex_racines():
    exo = ['\\exercice']
    exo.append('\\begin{enumerate}')
    exo.append(u'\\item Calculer les expressions suivantes et donner le résultat sous la forme $a\\,\\sqrt{b}$ avec $a$ et $b$ entiers, $b$ le plus petit possible.')
    exo.append('\\begin{multicols}{2}\\noindent')
    cor = ['\\exercice*']
    cor.append('\\begin{enumerate}')
    cor.append(u'\\item Calculer les expressions suivantes et donner le résultat sous la forme $a\\,\\sqrt{b}$ avec $a$ et $b$ entiers, $b$ le plus petit possible.')
    mymax = 5
    cor.append('\\begin{multicols}{2}\\noindent')
    valeurs = valeurs_aRb0(mymax)
    exoaRb0(exo, cor, valeurs)
    exo.append('\\columnbreak\\stepcounter{nocalcul}%')
    cor.append('\\columnbreak\\stepcounter{nocalcul}%')
    valeurs = valeurs_aRb1(mymax)
    exoaRb1(exo, cor, valeurs)
    exo.append('\\end{multicols}\\vspace{-3ex}')
    cor.append('\\end{multicols}')
    exo.append(u'\\item Calculer les expressions suivantes et donner le résultat sous la forme $a+b\\,\\sqrt{c}$ avec $a$, $b$ et $c$ entiers.')
    exo.append('''\\stepcounter{nocalcul}%
    \\begin{multicols}{2}\\noindent''')
    cor.append(u'\\item Calculer les expressions suivantes et donner le résultat sous la forme $a+b\\,\\sqrt{c}$ avec $a$, $b$ et $c$ entiers.')
    cor.append('''\\stepcounter{nocalcul}%
    \\begin{multicols}{2}\\noindent''')
    valeurs = valeurs_aPbRc(mymax)
    exo_aPbRc(exo, cor, valeurs)
    exo.append('\\columnbreak\\stepcounter{nocalcul}%')
    cor.append('\\columnbreak\\stepcounter{nocalcul}%')
    valeurs = valeurs_aPbRc(mymax)
    exo_aPbRc(exo, cor, valeurs)
    exo.append('\\end{multicols}\\vspace{-3ex}')
    cor.append('\\end{multicols}')
    exo.append(u"\\item Calculer les expressions suivantes et donner le résultat sous la forme d'un nombre entier.\n")
    exo.append('''\\stepcounter{nocalcul}%
    \\begin{multicols}{2}\\noindent''')
    cor.append(u"\\item Calculer les expressions suivantes et donner le résultat sous la forme d'un nombre entier.\n")
    cor.append('''\\stepcounter{nocalcul}%
    \\begin{multicols}{2}\\noindent''')
    valeurs = valeurs_entier0(mymax)
    exo_entier0(exo, cor, valeurs)
    exo.append('\\columnbreak\\stepcounter{nocalcul}%')
    cor.append('\\columnbreak\\stepcounter{nocalcul}%')
    valeurs = valeurs_entier1(mymax)
    exo_entier1(exo, cor, valeurs)
    exo.append('\\end{multicols}\\vspace{-3ex}')
    exo.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    return (exo, cor)

class tex_racines(LegacyExercise):
    """Racines carrées"""

    tags = ["Troisième"]
    function = _tex_racines



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
from math import atan, cos, pi, sin
from pyromaths.ex import LegacyExercise

#===============================================================================
# Symétrique d'une figure par rapport à une droite avec quadrillage
#===============================================================================


def valeurs_quad2(nb_pts):
    vals = []
    for i in range(nb_pts):
        angle = random.randrange(i * 360 // nb_pts, (i + 1) * 360 // nb_pts)
        vals.append(random.randrange(1, 7) / 2 * cos(angle * pi / 180), random.randrange(1, 7) / 2 * sin(angle * pi / 180))
    return vals


def valeurs_quad(nb_pts):
    vals = []
    for i in range(nb_pts):
        (alpha, beta) = ((i * 360) // nb_pts, ((i + 1) * 360) // nb_pts)
        (x, y, angle) = (0, 0, 0)
        while x == 0 or angle < alpha or angle > beta:
            (x, y) = (random.randrange(-6, 7) * .5, random.randrange(-6,
                      7) * .5)
            if x > 0:
                angle = int(atan(y / x) * 180 / pi + 360) % 360
            if x < 0:
                angle = int(atan(y / x) * 180 / pi + 180)
        vals.append((x, y))
    return vals


def centre_sym(vals):
    fin = 0
    while not fin:
        (fin, cpt) = (1, 0)
        (o1, o2) = (random.randrange(-6, 7) * .5, random.randrange(-6, 7) *
                    .5)
        while fin and cpt < len(vals):
            fin = fin and -3 <= 2 * o1 - vals[cpt][0] <= 3 and -3 <= 2 * \
                o2 - vals[cpt][1] <= 3
            cpt = cpt + 1
    return (o1, o2)


def place_pts(vals, O):
    txt = ["  \\pstGeonode[PointSymbol=none,PointName=none]"]
    for i in range(len(vals)):
        txt.append("(%s,%s)" % vals[i])
        txt.append("{%s}" % chr(i + 97))
    txt.append("\n  \\pstGeonode[PointSymbol=x, linecolor=Black, dotsize=6pt](%s,%s){O}" % O)
    txt.append("\n  \\pspolygon[linewidth=1pt]")
    for i in range(len(vals)):
        txt.append("(%s)" % chr(i + 97))
    return ("").join(txt)


def place_pts_sym(vals):
    txt = ["  \\pstSymO[PointSymbol=x,PointName=none]{O}{"]
    for i in range(len(vals)):
        if i > 0:
            txt.append(",")
        txt.append("%s" % chr(i + 97))
    txt.append("}[")
    for i in range(len(vals)):
        if i > 0:
            txt.append(",")
        txt.append("%s1" % chr(i + 97))
    txt.append("]\n\pspolygon[linecolor=Black,linestyle=dashed, linewidth=1pt]")
    for i in range(len(vals)):
        txt.append("(%s1)" % chr(i + 97))
    return ("").join(txt)


def exo_quadrillage(f0, f1):
    pass


def _symetrie():
    exo = ["\\exercice",
           _(u"Construire la symétrique de chacune des figures par rapport au point O en"),
           _("utilisant le quadrillage :\\par"), "\\psset{unit=.9cm}"]
    cor = ["\\exercice*",
           _(u"Construire la symétrique de chacune des figures par rapport au point O en"),
           _("utilisant le quadrillage :\\par"), "\\psset{unit=.9cm}"]
    nbpts = 5
    langles = [0, 90, 45, 135]
    for i in range(3):
        langles.pop(random.randrange(len(langles)))
        vals = valeurs_quad(nbpts)
        O = centre_sym(vals)
        txt = place_pts(vals, O)
        exo.append("\\begin{pspicture*}(-3,-3)(3,3)")
        exo.append("\\psgrid[subgriddiv=2,gridlabels=0pt]")
        exo.append(txt)
        cor.append("\\begin{pspicture*}(-3,-3)(3,3)")
        cor.append("\\psgrid[subgriddiv=2,gridlabels=0pt]")
        cor.append(txt)
        cor.append(place_pts_sym(vals))
        exo.append("\end{pspicture*}")
        cor.append("\end{pspicture*}")
        if i < 2:
            exo.append("\\hfill")
            cor.append("\\hfill")
    return (exo, cor)

class symetrie(LegacyExercise):
    """Symétrie centrale"""

    tags = ["Cinquième"]
    function = _symetrie



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from itertools import count
'''
Created on 24 janv. 2015

@author: jerome
'''
from pyromaths import ex
from pyromaths.outils.Arithmetique import pgcd
from pyromaths.classes.Fractions import Fraction
from random import randrange, shuffle

def InitPoints(minimum=-6.1, maximum=6.1, nbval=3):
    dY = []
    directions = [(-1) ** i for i in range(nbval - 1)]
    directions.append(0)
    shuffle(directions)
    for i in range((nbval - 1) // 2):
        while True:
            a, b = randrange(1, 5), randrange(1, 5)
            if pgcd(a, b) == 1 and a % b != 0:
                dY.append(Fraction(a, b))
                break
    for i in range(nbval - 1 - len(dY)):
        dY.append(randrange(1, 5))
    shuffle(dY)
    for i in range(nbval):
        if directions[i] == 0:
            dY.insert(i, 0)
        else:
            dY[i] = dY[i] * directions[i]
    dY.insert(0, 0)
    dY.append(0)
    redo = True
    while redo:
        lX = [int(minimum) + 1 + int(1.*i * (maximum - minimum - 2) / nbval) + randrange(4) for i in range(nbval)]
        for i in range(len(lX) - 1):
            if lX[i + 1:].count(lX[i]):
                redo = True
                break
            else:
                redo = False
    lX.insert(0, minimum)
    lX.append(maximum)
    lX.sort()
    lY = [randrange(-4, 5)]
    for i in range(1, len(lX)):
        inf = lY[-1] - 4 * int(lX[i] - lX[i - 1])
        sup = lY[-1] + 4 * int(lX[i] - lX[i - 1])
        nY = randrange(int(max(-4, inf)), int(min(5, sup)))
        lY.append(nY)
        #=======================================================================
        # while True:
        #     lg = randrange(1, 4) * (-1) ** randrange(2) * int(lX[i] - lX[i - 1])
        #     nY = lY[-1] + lg
        #     if -4 <= nY <= 4 :
        #         lY.append(nY)
        #         break
        #=======================================================================
    return lX, lY, dY


class Fd1Tangentes(ex.TexExercise):
    """Nombre dérivé graphiquement"""

    tags = ["1re Spé math"]


    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.lX1, self.lY1, self.dY1 = InitPoints(minimum=-6.1, maximum=6.1, nbval=3)
        self.lX2, self.lY2, self.dY2 = InitPoints(minimum=-6.1, maximum=6.1, nbval=4)


    def tex_statement(self):
        dY1 = list(self.dY1)
        for i in range(len(dY1)):
            if isinstance(dY1[i], Fraction): dY1[i] = 1. * dY1[i].n / dY1[i].d
        exo = [r'\exercice']
        exo.append(r'\begin{minipage}[]{\linewidth-8cm}')
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Déterminer graphiquement les nombres dérivés de la fonction $f$ en $\qquad x=%s \qquad x=%s \qquad x=%s$.') % (self.lX1[1], self.lX1[2], self.lX1[3]))
        exo.append(_(u'\\item On considère le tableau de valeurs suivant :\\par'))
        exo.append(r'\renewcommand{\arraystretch}{2}')
        exo.append(r'\begin{tabularx}{\linewidth}[t]{|*5{>{\centering\arraybackslash}X|}}')
        exo.append(r'\hline')
        exo.append(r'$x$ & $%s$ \\ \hline' % ("$ & $".join([str(a) for a in self.lX2[1:-1]])))
        exo.append(r'$g\,(x)$ & $%s$ \\ \hline' % ("$ & $".join([str(a) for a in self.lY2[1:-1]])))
        exo.append('$g\'\\,(x)$ & $%s$ \\\\ \\hline' % ("$ & $".join([str(a) for a in self.dY2[1:-1]])))
        exo.append(r'\end{tabularx}')
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item Dans un nouveau repère, placer les points de la courbe $\\mathcal{C}_g$ ainsi connus.'))
        exo.append(_(u'\\item Tracer les tangentes à $\\mathcal{C}_g$ en ces points.'))
        exo.append(_(u'\\item Donner une allure possible de la courbe $\\mathcal{C}_g$.'))
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{enumerate}')
        exo.append(r'\end{minipage}')
        exo.append(r'\hfill')
        exo.append(r'\begin{minipage}[]{7.5cm}')
        exo.append(r'\begin{asy}[width=\linewidth]')
        exo.append(r'import graph;')
        exo.append(r'import interpolate;')
        exo.append(r'import geometry;')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt={%s};' % (",".join([str(a).zfill(2) for a in self.lX1])))
        exo.append(r'real[] ypt={%s};' % (",".join([str(a).zfill(2) for a in self.lY1])))
        exo.append(r'real[] dy= {%s};' % (",".join([str(a).zfill(2) for a in dY1])))
        exo.append(r'real f(real t){return pwhermite(xpt,ypt,dy)(t);}')
        exo.append(r'path Cf=graph(f,-6.1,6.1);')
        exo.append(r'void tangente(int k,real lg=sqrt(1+dy[k]^2),real ld=lg, pen p=black+1, arrowbar arr=Arrows(SimpleHead,size=9pt)) {')
        exo.append(r'draw(((xpt[k],ypt[k])-lg*unit((1,dy[k])))--((xpt[k],ypt[k])+ld*unit((1,dy[k]))),p,arr);')
        exo.append(r'dot((xpt[k],ypt[k]));')
        exo.append(r'}')
        exo.append(r'xlimits(-6.1, 6.1);')
        exo.append(r'ylimits(-5.5, 5.5, Crop);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5pt, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5pt, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label("$\mathcal C_f$", (-6, f(-6)), 1.5%sE, brown);' % ('NS'[self.lY1[1] > self.lY1[0]]))
        exo.append(r'draw(Cf, brown+1.5);')
        exo.append(r'tangente(1,lg=sqrt(%s));' % (Fraction(self.dY1[1]).n ** 2 + Fraction(self.dY1[1]).d ** 2))
        exo.append(r'tangente(2,lg=sqrt(%s));' % (Fraction(self.dY1[2]).n ** 2 + Fraction(self.dY1[2]).d ** 2))
        exo.append(r'tangente(3,lg=sqrt(%s));' % (Fraction(self.dY1[3]).n ** 2 + Fraction(self.dY1[3]).d ** 2))
        exo.append(r'xlimits(-6.1, 6.1, Crop);')
        exo.append(r'ylimits(-5.5, 5.5, Crop);')
        exo.append(r'\end{asy}')
        exo.append(r'\end{minipage}')
        return "\n".join(exo)

    def tex_answer(self):
        dY2 = list(self.dY2)
        for i in range(len(dY2)):
            if isinstance(dY2[i], Fraction): dY2[i] = 1. * dY2[i].n / dY2[i].d
        exo = [r'\exercice*']
        exo.append(r'\begin{enumerate}')
        exo.append(_(u'\\item  On lit graphiquement le coefficient directeur de chacune des tangentes en ces points.\par'))
        exo.append(u'$f\'\\,(%s)=%s \\qquad f\'\\,(%s)=%s \\qquad f\'\\,(%s)=%s$.' % (self.lX1[1], self.dY1[1], self.lX1[2], self.dY1[2], self.lX1[3], self.dY1[3]))
        exo.append(u'\\item')
        exo.append(r'\begin{asy}[height=6.5cm]')
        exo.append(r'import graph;')
        exo.append(r'import interpolate;')
        exo.append(r'import geometry;')
        exo.append(r'defaultpen(fontsize(9pt));')
        exo.append(r'real[] xpt={%s};' % (",".join([str(a).zfill(2) for a in self.lX2])))
        exo.append(r'real[] ypt={%s};' % (",".join([str(a).zfill( 2) for a in self.lY2])))
        exo.append(r'real[] dy= {%s};' % (",".join([str(a).zfill(2) for a in dY2])))
        exo.append(r'real f(real t){return pwhermite(xpt,ypt,dy)(t);}')
        exo.append(r'path Cf=graph(f,-6.1,6.1);')
        exo.append(r'void tangente(int k,real lg=sqrt(1+dy[k]^2),real ld=lg, pen p=black+1, arrowbar arr=Arrows(SimpleHead,size=9pt)) {')
        exo.append(r'draw(((xpt[k],ypt[k])-lg*unit((1,dy[k])))--((xpt[k],ypt[k])+ld*unit((1,dy[k]))),p,arr);')
        exo.append(r'dot((xpt[k],ypt[k]));')
        exo.append(r'}')
        exo.append(r'xlimits(%s, %s);' % (self.lX2[1] - 1.1, self.lX2[-2] + 1.1))
        exo.append(r'ylimits(-5.5, 5.5, Crop);')
        exo.append(r'xaxis(axis=BottomTop, p=invisible,')
        exo.append(r'ticks=Ticks(format="%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5pt, ptick=dotted)')
        exo.append(r');')
        exo.append(r'yaxis(axis=LeftRight, p=invisible,')
        exo.append(r'ticks=Ticks(format="%", Step=1, extend=true,')
        exo.append(r'pTick=gray+.5pt, ptick=dotted)')
        exo.append(r');')
        exo.append(r'xequals(L="$y$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'yequals(L="$x$", 0, extend=false, arrow=Arrow(HookHead, size=9pt), p=black+1,')
        exo.append(r'ticks=Ticks(scale(.7)*Label(filltype=Fill(white)), Step=1, Size=3pt,')
        exo.append(r'end=false, endlabel=false, beginlabel=false, NoZero));')
        exo.append(r'labelx(L=scale(.7)*"$0$", (0,0), align=SW);')
        exo.append(r'label("$\mathcal C_f$", (-6, f(-6)), 1.5%sE, brown);' % ('NS'[self.lY2[1] > self.lY2[0]]))
        exo.append(r'draw(Cf, brown+1);')
        exo.append(r'tangente(1,lg=sqrt(%s));' % (Fraction(self.dY2[1]).n ** 2 + Fraction(self.dY2[1]).d ** 2))
        exo.append(r'tangente(2,lg=sqrt(%s));' % (Fraction(self.dY2[2]).n ** 2 + Fraction(self.dY2[2]).d ** 2))
        exo.append(r'tangente(3,lg=sqrt(%s));' % (Fraction(self.dY2[3]).n ** 2 + Fraction(self.dY2[3]).d ** 2))
        exo.append(r'tangente(4,lg=sqrt(%s));' % (Fraction(self.dY2[4]).n ** 2 + Fraction(self.dY2[4]).d ** 2))
        exo.append(r'xlimits(%s, %s, Crop);' % (self.lX2[1] - 1.1, self.lX2[-2] + 1.1))
        exo.append(r'ylimits(-5.5, 5.5, Crop);')
        exo.append(r'\end{asy}')
        exo.append(r'\end{enumerate}')
        return "\n".join(exo)
